# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2012 Isaku Yamahata <yamahata at private email ne jp>
#                               <yamahata at valinux co jp>
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
# @author: Isaku Yamahata

from oslo.config import cfg
from ryu.app import client
from ryu.app import rest_nw_id

from neutron.agent import securitygroups_rpc as sg_rpc
from neutron.common import constants as q_const
from neutron.common import exceptions as n_exc
from neutron.common import rpc as q_rpc
from neutron.common import topics
from neutron.db import api as db
from neutron.db import db_base_plugin_v2
from neutron.db import dhcp_rpc_base
from neutron.db import external_net_db
from neutron.db import extraroute_db
from neutron.db import l3_gwmode_db
from neutron.db import l3_rpc_base
from neutron.db import models_v2
from neutron.db import portbindings_base
from neutron.db import securitygroups_rpc_base as sg_db_rpc
from neutron.extensions import portbindings
from neutron.openstack.common import excutils
from neutron.openstack.common import log as logging
from neutron.openstack.common import rpc
from neutron.openstack.common.rpc import proxy
from neutron.plugins.common import constants as svc_constants
from neutron.plugins.ryu.common import config  # noqa
from neutron.plugins.ryu.db import api_v2 as db_api_v2


LOG = logging.getLogger(__name__)


class RyuRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin,
                      l3_rpc_base.L3RpcCallbackMixin,
                      sg_db_rpc.SecurityGroupServerRpcCallbackMixin):

    RPC_API_VERSION = '1.1'

    def __init__(self, ofp_rest_api_addr):
        self.ofp_rest_api_addr = ofp_rest_api_addr

    def create_rpc_dispatcher(self):
        return q_rpc.PluginRpcDispatcher([self])

    def get_ofp_rest_api(self, context, **kwargs):
        LOG.debug(_("get_ofp_rest_api: %s"), self.ofp_rest_api_addr)
        return self.ofp_rest_api_addr

    @classmethod
    def get_port_from_device(cls, device):
        port = db_api_v2.get_port_from_device(device)
        if port:
            port['device'] = device
        return port


class AgentNotifierApi(proxy.RpcProxy,
                       sg_rpc.SecurityGroupAgentRpcApiMixin):

    BASE_RPC_API_VERSION = '1.0'

    def __init__(self, topic):
        super(AgentNotifierApi, self).__init__(
            topic=topic, default_version=self.BASE_RPC_API_VERSION)
        self.topic_port_update = topics.get_topic_name(topic,
                                                       topics.PORT,
                                                       topics.UPDATE)

    def port_update(self, context, port):
        self.fanout_cast(context,
                         self.make_msg('port_update', port=port),
                         topic=self.topic_port_update)


class RyuNeutronPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                         external_net_db.External_net_db_mixin,
                         extraroute_db.ExtraRoute_db_mixin,
                         l3_gwmode_db.L3_NAT_db_mixin,
                         sg_db_rpc.SecurityGroupServerRpcMixin,
                         portbindings_base.PortBindingBaseMixin):

    _supported_extension_aliases = ["external-net", "router", "ext-gw-mode",
                                    "extraroute", "security-group",
                                    "binding", "quotas"]

    @property
    def supported_extension_aliases(self):
        if not hasattr(self, '_aliases'):
            aliases = self._supported_extension_aliases[:]
            sg_rpc.disable_security_group_extension_by_config(aliases)
            self._aliases = aliases
        return self._aliases

    def __init__(self, configfile=None):
        super(RyuNeutronPluginV2, self).__init__()
        self.base_binding_dict = {
            portbindings.VIF_TYPE: portbindings.VIF_TYPE_OVS,
            portbindings.VIF_DETAILS: {
                # TODO(rkukura): Replace with new VIF security details
                portbindings.CAP_PORT_FILTER:
                'security-group' in self.supported_extension_aliases,
                portbindings.OVS_HYBRID_PLUG: True
            }
        }
        portbindings_base.register_port_dict_function()
        self.tunnel_key = db_api_v2.TunnelKey(
            cfg.CONF.OVS.tunnel_key_min, cfg.CONF.OVS.tunnel_key_max)
        self.ofp_api_host = cfg.CONF.OVS.openflow_rest_api
        if not self.ofp_api_host:
            raise n_exc.Invalid(_('Invalid configuration. check ryu.ini'))

        self.client = client.OFPClient(self.ofp_api_host)
        self.tun_client = client.TunnelClient(self.ofp_api_host)
        self.iface_client = client.NeutronIfaceClient(self.ofp_api_host)
        for nw_id in rest_nw_id.RESERVED_NETWORK_IDS:
            if nw_id != rest_nw_id.NW_ID_UNKNOWN:
                self.client.update_network(nw_id)
        self._setup_rpc()

        # register known all network list on startup
        self._create_all_tenant_network()

    def _setup_rpc(self):
        self.service_topics = {svc_constants.CORE: topics.PLUGIN,
                               svc_constants.L3_ROUTER_NAT: topics.L3PLUGIN}
        self.conn = rpc.create_connection(new=True)
        self.notifier = AgentNotifierApi(topics.AGENT)
        self.callbacks = RyuRpcCallbacks(self.ofp_api_host)
        self.dispatcher = self.callbacks.create_rpc_dispatcher()
        for svc_topic in self.service_topics.values():
            self.conn.create_consumer(svc_topic, self.dispatcher, fanout=False)
        self.conn.consume_in_thread()

    def _create_all_tenant_network(self):
        for net in db_api_v2.network_all_tenant_list():
            self.client.update_network(net.id)
        for tun in self.tunnel_key.all_list():
            self.tun_client.update_tunnel_key(tun.network_id, tun.tunnel_key)
        session = db.get_session()
        for port in session.query(models_v2.Port):
            self.iface_client.update_network_id(port.id, port.network_id)

    def _client_create_network(self, net_id, tunnel_key):
        self.client.create_network(net_id)
        self.tun_client.create_tunnel_key(net_id, tunnel_key)

    def _client_delete_network(self, net_id):
        client.ignore_http_not_found(
            lambda: self.client.delete_network(net_id))
        client.ignore_http_not_found(
            lambda: self.tun_client.delete_tunnel_key(net_id))

    def create_network(self, context, network):
        session = context.session
        with session.begin(subtransactions=True):
            #set up default security groups
            tenant_id = self._get_tenant_id_for_create(
                context, network['network'])
            self._ensure_default_security_group(context, tenant_id)

            net = super(RyuNeutronPluginV2, self).create_network(context,
                                                                 network)
            self._process_l3_create(context, net, network['network'])

            tunnel_key = self.tunnel_key.allocate(session, net['id'])
            try:
                self._client_create_network(net['id'], tunnel_key)
            except Exception:
                with excutils.save_and_reraise_exception():
                    self._client_delete_network(net['id'])

        return net

    def update_network(self, context, id, network):
        session = context.session
        with session.begin(subtransactions=True):
            net = super(RyuNeutronPluginV2, self).update_network(context, id,
                                                                 network)
            self._process_l3_update(context, net, network['network'])
        return net

    def delete_network(self, context, id):
        self._client_delete_network(id)
        session = context.session
        with session.begin(subtransactions=True):
            self.tunnel_key.delete(session, id)
            super(RyuNeutronPluginV2, self).delete_network(context, id)

    def create_port(self, context, port):
        session = context.session
        port_data = port['port']
        with session.begin(subtransactions=True):
            self._ensure_default_security_group_on_port(context, port)
            sgids = self._get_security_groups_on_port(context, port)
            port = super(RyuNeutronPluginV2, self).create_port(context, port)
            self._process_portbindings_create_and_update(context,
                                                         port_data,
                                                         port)
            self._process_port_create_security_group(
                context, port, sgids)
        self.notify_security_groups_member_updated(context, port)
        self.iface_client.create_network_id(port['id'], port['network_id'])
        return port

    def delete_port(self, context, id, l3_port_check=True):
        # if needed, check to see if this is a port owned by
        # and l3-router. If so, we should prevent deletion.
        if l3_port_check:
            self.prevent_l3_port_deletion(context, id)

        with context.session.begin(subtransactions=True):
            self.disassociate_floatingips(context, id)
            port = self.get_port(context, id)
            self._delete_port_security_group_bindings(context, id)
            super(RyuNeutronPluginV2, self).delete_port(context, id)

        self.notify_security_groups_member_updated(context, port)

    def update_port(self, context, id, port):
        deleted = port['port'].get('deleted', False)
        session = context.session

        need_port_update_notify = False
        with session.begin(subtransactions=True):
            original_port = super(RyuNeutronPluginV2, self).get_port(
                context, id)
            updated_port = super(RyuNeutronPluginV2, self).update_port(
                context, id, port)
            self._process_portbindings_create_and_update(context,
                                                         port['port'],
                                                         updated_port)
            need_port_update_notify = self.update_security_group_on_port(
                context, id, port, original_port, updated_port)

        need_port_update_notify |= self.is_security_group_member_updated(
            context, original_port, updated_port)

        need_port_update_notify |= (original_port['admin_state_up'] !=
                                    updated_port['admin_state_up'])

        if need_port_update_notify:
            self.notifier.port_update(context, updated_port)

        if deleted:
            db_api_v2.set_port_status(session, id, q_const.PORT_STATUS_DOWN)
        return updated_port



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""initial port security

Revision ID: 1149d7de0cfa
Revises: 1b693c095aa3
Create Date: 2013-01-22 14:05:20.696502

"""

# revision identifiers, used by Alembic.
revision = '1149d7de0cfa'
down_revision = '1b693c095aa3'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = [
    'neutron.plugins.nicira.NeutronPlugin.NvpPluginV2',
    'neutron.plugins.nicira.NeutronServicePlugin.NvpAdvancedPlugin',
    'neutron.plugins.vmware.plugin.NsxPlugin',
    'neutron.plugins.vmware.plugin.NsxServicePlugin'
]

from alembic import op
import sqlalchemy as sa

from neutron.db import migration


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('networksecuritybindings',
                    sa.Column('network_id', sa.String(length=36),
                              nullable=False),
                    sa.Column('port_security_enabled', sa.Boolean(),
                              nullable=False),
                    sa.ForeignKeyConstraint(['network_id'], ['networks.id'],
                                            ondelete='CASCADE'),
                    sa.PrimaryKeyConstraint('network_id'))
    op.create_table('portsecuritybindings',
                    sa.Column('port_id', sa.String(length=36),
                              nullable=False),
                    sa.Column('port_security_enabled', sa.Boolean(),
                              nullable=False),
                    sa.ForeignKeyConstraint(['port_id'], ['ports.id'],
                                            ondelete='CASCADE'),
                    sa.PrimaryKeyConstraint('port_id'))
    ### end Alembic commands ###

    # Copy network and port ids over to network|port(securitybindings) table
    # and set port_security_enabled to false as ip address pairs were not
    # configured in NVP/NSX originally.
    op.execute("INSERT INTO networksecuritybindings SELECT id as "
               "network_id, False as port_security_enabled from networks")
    op.execute("INSERT INTO portsecuritybindings SELECT id as port_id, "
               "False as port_security_enabled from ports")


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('portsecuritybindings')
    op.drop_table('networksecuritybindings')
    ### end Alembic commands ###



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013, Nachi Ueno, NTT MCL, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
from oslo.config import cfg
import sqlalchemy as sa
from sqlalchemy import orm

from neutron.common import utils
from neutron.db import db_base_plugin_v2
from neutron.db import l3_db
from neutron.db import model_base
from neutron.db import models_v2
from neutron.extensions import extraroute
from neutron.extensions import l3
from neutron.openstack.common import log as logging


LOG = logging.getLogger(__name__)

extra_route_opts = [
    #TODO(nati): use quota framework when it support quota for attributes
    cfg.IntOpt('max_routes', default=30,
               help=_("Maximum number of routes")),
]

cfg.CONF.register_opts(extra_route_opts)


class RouterRoute(model_base.BASEV2, models_v2.Route):
    router_id = sa.Column(sa.String(36),
                          sa.ForeignKey('routers.id',
                                        ondelete="CASCADE"),
                          primary_key=True)

    router = orm.relationship(l3_db.Router,
                              backref=orm.backref("route_list",
                                                  lazy='joined',
                                                  cascade='delete'))


class ExtraRoute_db_mixin(l3_db.L3_NAT_db_mixin):
    """Mixin class to support extra route configuration on router."""

    def _extend_router_dict_extraroute(self, router_res, router_db):
        router_res['routes'] = (ExtraRoute_db_mixin.
                                _make_extra_route_list(
                                    router_db['route_list']
                                ))

    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
        l3.ROUTERS, ['_extend_router_dict_extraroute'])

    def update_router(self, context, id, router):
        r = router['router']
        with context.session.begin(subtransactions=True):
            #check if route exists and have permission to access
            router_db = self._get_router(context, id)
            if 'routes' in r:
                self._update_extra_routes(context, router_db, r['routes'])
            routes = self._get_extra_routes_by_router_id(context, id)
        router_updated = super(ExtraRoute_db_mixin, self).update_router(
            context, id, router)
        router_updated['routes'] = routes

        return router_updated

    def _get_subnets_by_cidr(self, context, cidr):
        query_subnets = context.session.query(models_v2.Subnet)
        return query_subnets.filter_by(cidr=cidr).all()

    def _validate_routes_nexthop(self, context, ports, routes, nexthop):
        #Note(nati): Nexthop should be connected,
        # so we need to check
        # nexthop belongs to one of cidrs of the router ports
        cidrs = []
        for port in ports:
            cidrs += [self._core_plugin._get_subnet(context,
                                                    ip['subnet_id'])['cidr']
                      for ip in port['fixed_ips']]
        if not netaddr.all_matching_cidrs(nexthop, cidrs):
            raise extraroute.InvalidRoutes(
                routes=routes,
                reason=_('the nexthop is not connected with router'))
        #Note(nati) nexthop should not be same as fixed_ips
        for port in ports:
            for ip in port['fixed_ips']:
                if nexthop == ip['ip_address']:
                    raise extraroute.InvalidRoutes(
                        routes=routes,
                        reason=_('the nexthop is used by router'))

    def _validate_routes(self, context,
                         router_id, routes):
        if len(routes) > cfg.CONF.max_routes:
            raise extraroute.RoutesExhausted(
                router_id=router_id,
                quota=cfg.CONF.max_routes)

        filters = {'device_id': [router_id]}
        ports = self._core_plugin.get_ports(context, filters)
        for route in routes:
            self._validate_routes_nexthop(
                context, ports, routes, route['nexthop'])

    def _update_extra_routes(self, context, router, routes):
        self._validate_routes(context, router['id'],
                              routes)
        old_routes = self._get_extra_routes_by_router_id(
            context, router['id'])
        added, removed = utils.diff_list_of_dict(old_routes,
                                                 routes)
        LOG.debug(_('Added routes are %s'), added)
        for route in added:
            router_routes = RouterRoute(
                router_id=router['id'],
                destination=route['destination'],
                nexthop=route['nexthop'])
            context.session.add(router_routes)

        LOG.debug(_('Removed routes are %s'), removed)
        for route in removed:
            del_context = context.session.query(RouterRoute)
            del_context.filter_by(router_id=router['id'],
                                  destination=route['destination'],
                                  nexthop=route['nexthop']).delete()

    @staticmethod
    def _make_extra_route_list(extra_routes):
        return [{'destination': route['destination'],
                 'nexthop': route['nexthop']}
                for route in extra_routes]

    def _get_extra_routes_by_router_id(self, context, id):
        query = context.session.query(RouterRoute)
        query = query.filter_by(router_id=id)
        return self._make_extra_route_list(query)

    def get_router(self, context, id, fields=None):
        with context.session.begin(subtransactions=True):
            router = super(ExtraRoute_db_mixin, self).get_router(
                context, id, fields)
            return router

    def get_routers(self, context, filters=None, fields=None,
                    sorts=None, limit=None, marker=None,
                    page_reverse=False):
        with context.session.begin(subtransactions=True):
            routers = super(ExtraRoute_db_mixin, self).get_routers(
                context, filters, fields, sorts=sorts, limit=limit,
                marker=marker, page_reverse=page_reverse)
            return routers

    def _confirm_router_interface_not_in_use(self, context, router_id,
                                             subnet_id):
        super(ExtraRoute_db_mixin, self)._confirm_router_interface_not_in_use(
            context, router_id, subnet_id)
        subnet_db = self._core_plugin._get_subnet(context, subnet_id)
        subnet_cidr = netaddr.IPNetwork(subnet_db['cidr'])
        extra_routes = self._get_extra_routes_by_router_id(context, router_id)
        for route in extra_routes:
            if netaddr.all_matching_cidrs(route['nexthop'], [subnet_cidr]):
                raise extraroute.RouterInterfaceInUseByRoute(
                    router_id=router_id, subnet_id=subnet_id)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import eventlet

from neutron.agent.linux import async_process
from neutron.openstack.common import log as logging


LOG = logging.getLogger(__name__)


class OvsdbMonitor(async_process.AsyncProcess):
    """Manages an invocation of 'ovsdb-client monitor'."""

    def __init__(self, table_name, columns=None, format=None,
                 root_helper=None, respawn_interval=None):

        cmd = ['ovsdb-client', 'monitor', table_name]
        if columns:
            cmd.append(','.join(columns))
        if format:
            cmd.append('--format=%s' % format)
        super(OvsdbMonitor, self).__init__(cmd,
                                           root_helper=root_helper,
                                           respawn_interval=respawn_interval)

    def _read_stdout(self):
        data = self._process.stdout.readline()
        if not data:
            return
        #TODO(marun) The default root helper outputs exit errors to
        # stdout due to bug #1219530.  This check can be moved to
        # _read_stderr once the error is correctly output to stderr.
        if self.root_helper and self.root_helper in data:
            self._stderr_lines.put(data)
            LOG.error(_('Error received from ovsdb monitor: %s') % data)
        else:
            self._stdout_lines.put(data)
            LOG.debug(_('Output received from ovsdb monitor: %s') % data)
            return data

    def _read_stderr(self):
        data = super(OvsdbMonitor, self)._read_stderr()
        if data:
            LOG.error(_('Error received from ovsdb monitor: %s') % data)
            # Do not return value to ensure that stderr output will
            # stop the monitor.


class SimpleInterfaceMonitor(OvsdbMonitor):
    """Monitors the Interface table of the local host's ovsdb for changes.

    The has_updates() method indicates whether changes to the ovsdb
    Interface table have been detected since the monitor started or
    since the previous access.
    """

    def __init__(self, root_helper=None, respawn_interval=None):
        super(SimpleInterfaceMonitor, self).__init__(
            'Interface',
            columns=['name', 'ofport'],
            format='json',
            root_helper=root_helper,
            respawn_interval=respawn_interval,
        )
        self.data_received = False

    @property
    def is_active(self):
        return (self.data_received and
                self._kill_event and
                not self._kill_event.ready())

    @property
    def has_updates(self):
        """Indicate whether the ovsdb Interface table has been updated.

        True will be returned if the monitor process is not active.
        This 'failing open' minimizes the risk of falsely indicating
        the absence of updates at the expense of potential false
        positives.
        """
        return bool(list(self.iter_stdout())) or not self.is_active

    def start(self, block=False, timeout=5):
        super(SimpleInterfaceMonitor, self).start()
        if block:
            eventlet.timeout.Timeout(timeout)
            while not self.is_active:
                eventlet.sleep()

    def _kill(self, *args, **kwargs):
        self.data_received = False
        super(SimpleInterfaceMonitor, self)._kill(*args, **kwargs)

    def _read_stdout(self):
        data = super(SimpleInterfaceMonitor, self)._read_stdout()
        if data and not self.data_received:
            self.data_received = True
        return data



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
IPv6-related utilities and helper functions.
"""

import netaddr


def get_ipv6_addr_by_EUI64(prefix, mac):
    # Check if the prefix is IPv4 address
    isIPv4 = netaddr.valid_ipv4(prefix)
    if isIPv4:
        msg = _("Unable to generate IP address by EUI64 for IPv4 prefix")
        raise TypeError(msg)
    try:
        eui64 = int(netaddr.EUI(mac).eui64())
        prefix = netaddr.IPNetwork(prefix)
        return netaddr.IPAddress(prefix.first + eui64 ^ (1 << 57))
    except netaddr.AddrFormatError:
        raise TypeError(_('Bad prefix or mac format for generating IPv6 '
                          'address by EUI-64: %(prefix)s, %(mac)s:')
                        % {'prefix': prefix, 'mac': mac})
    except TypeError:
        raise TypeError(_('Bad prefix type for generate IPv6 address by '
                          'EUI-64: %s') % prefix)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 New Dream Network, LLC (DreamHost)
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Mark McClain, DreamHost

import itertools
from six.moves import xrange

from neutron.agent.linux import utils
from neutron.plugins.common import constants as qconstants
from neutron.services.loadbalancer import constants


PROTOCOL_MAP = {
    constants.PROTOCOL_TCP: 'tcp',
    constants.PROTOCOL_HTTP: 'http',
    constants.PROTOCOL_HTTPS: 'tcp',
}

BALANCE_MAP = {
    constants.LB_METHOD_ROUND_ROBIN: 'roundrobin',
    constants.LB_METHOD_LEAST_CONNECTIONS: 'leastconn',
    constants.LB_METHOD_SOURCE_IP: 'source'
}

STATS_MAP = {
    constants.STATS_ACTIVE_CONNECTIONS: 'qcur',
    constants.STATS_MAX_CONNECTIONS: 'qmax',
    constants.STATS_CURRENT_SESSIONS: 'scur',
    constants.STATS_MAX_SESSIONS: 'smax',
    constants.STATS_TOTAL_SESSIONS: 'stot',
    constants.STATS_IN_BYTES: 'bin',
    constants.STATS_OUT_BYTES: 'bout',
    constants.STATS_CONNECTION_ERRORS: 'econ',
    constants.STATS_RESPONSE_ERRORS: 'eresp'
}

ACTIVE_PENDING_STATUSES = qconstants.ACTIVE_PENDING_STATUSES
INACTIVE = qconstants.INACTIVE


def save_config(conf_path, logical_config, socket_path=None,
                user_group='nogroup'):
    """Convert a logical configuration to the HAProxy version."""
    data = []
    data.extend(_build_global(logical_config, socket_path=socket_path,
                              user_group=user_group))
    data.extend(_build_defaults(logical_config))
    data.extend(_build_frontend(logical_config))
    data.extend(_build_backend(logical_config))
    utils.replace_file(conf_path, '\n'.join(data))


def _build_global(config, socket_path=None, user_group='nogroup'):
    opts = [
        'daemon',
        'user nobody',
        'group %s' % user_group,
        'log /dev/log local0',
        'log /dev/log local1 notice'
    ]

    if socket_path:
        opts.append('stats socket %s mode 0666 level user' % socket_path)

    return itertools.chain(['global'], ('\t' + o for o in opts))


def _build_defaults(config):
    opts = [
        'log global',
        'retries 3',
        'option redispatch',
        'timeout connect 5000',
        'timeout client 50000',
        'timeout server 50000',
    ]

    return itertools.chain(['defaults'], ('\t' + o for o in opts))


def _build_frontend(config):
    protocol = config['vip']['protocol']

    opts = [
        'option tcplog',
        'bind %s:%d' % (
            _get_first_ip_from_port(config['vip']['port']),
            config['vip']['protocol_port']
        ),
        'mode %s' % PROTOCOL_MAP[protocol],
        'default_backend %s' % config['pool']['id'],
    ]

    if config['vip']['connection_limit'] >= 0:
        opts.append('maxconn %s' % config['vip']['connection_limit'])

    if protocol == constants.PROTOCOL_HTTP:
        opts.append('option forwardfor')

    return itertools.chain(
        ['frontend %s' % config['vip']['id']],
        ('\t' + o for o in opts)
    )


def _build_backend(config):
    protocol = config['pool']['protocol']
    lb_method = config['pool']['lb_method']

    opts = [
        'mode %s' % PROTOCOL_MAP[protocol],
        'balance %s' % BALANCE_MAP.get(lb_method, 'roundrobin')
    ]

    if protocol == constants.PROTOCOL_HTTP:
        opts.append('option forwardfor')

    # add the first health_monitor (if available)
    server_addon, health_opts = _get_server_health_option(config)
    opts.extend(health_opts)

    # add session persistence (if available)
    persist_opts = _get_session_persistence(config)
    opts.extend(persist_opts)

    # add the members
    for member in config['members']:
        if ((member['status'] in ACTIVE_PENDING_STATUSES or
             member['status'] == INACTIVE)
            and member['admin_state_up']):
            server = (('server %(id)s %(address)s:%(protocol_port)s '
                       'weight %(weight)s') % member) + server_addon
            if _has_http_cookie_persistence(config):
                server += ' cookie %d' % config['members'].index(member)
            opts.append(server)

    return itertools.chain(
        ['backend %s' % config['pool']['id']],
        ('\t' + o for o in opts)
    )


def _get_first_ip_from_port(port):
    for fixed_ip in port['fixed_ips']:
        return fixed_ip['ip_address']


def _get_server_health_option(config):
    """return the first active health option."""
    for monitor in config['healthmonitors']:
        # not checking the status of healthmonitor for two reasons:
        # 1) status field is absent in HealthMonitor model
        # 2) only active HealthMonitors are fetched with
        # LoadBalancerCallbacks.get_logical_device
        if monitor['admin_state_up']:
            break
    else:
        return '', []

    server_addon = ' check inter %(delay)ds fall %(max_retries)d' % monitor
    opts = [
        'timeout check %ds' % monitor['timeout']
    ]

    if monitor['type'] in (constants.HEALTH_MONITOR_HTTP,
                           constants.HEALTH_MONITOR_HTTPS):
        opts.append('option httpchk %(http_method)s %(url_path)s' % monitor)
        opts.append(
            'http-check expect rstatus %s' %
            '|'.join(_expand_expected_codes(monitor['expected_codes']))
        )

    if monitor['type'] == constants.HEALTH_MONITOR_HTTPS:
        opts.append('option ssl-hello-chk')

    return server_addon, opts


def _get_session_persistence(config):
    persistence = config['vip'].get('session_persistence')
    if not persistence:
        return []

    opts = []
    if persistence['type'] == constants.SESSION_PERSISTENCE_SOURCE_IP:
        opts.append('stick-table type ip size 10k')
        opts.append('stick on src')
    elif persistence['type'] == constants.SESSION_PERSISTENCE_HTTP_COOKIE:
        opts.append('cookie SRV insert indirect nocache')
    elif (persistence['type'] == constants.SESSION_PERSISTENCE_APP_COOKIE and
          persistence.get('cookie_name')):
        opts.append('appsession %s len 56 timeout 3h' %
                    persistence['cookie_name'])

    return opts


def _has_http_cookie_persistence(config):
    return (config['vip'].get('session_persistence') and
            config['vip']['session_persistence']['type'] ==
            constants.SESSION_PERSISTENCE_HTTP_COOKIE)


def _expand_expected_codes(codes):
    """Expand the expected code string in set of codes.

    200-204 -> 200, 201, 202, 204
    200, 203 -> 200, 203
    """

    retval = set()
    for code in codes.replace(',', ' ').split(' '):
        code = code.strip()

        if not code:
            continue
        elif '-' in code:
            low, hi = code.split('-')[:2]
            retval.update(str(i) for i in xrange(int(low), int(hi) + 1))
        else:
            retval.add(code)
    return retval



# Copyright 2014 Citrix Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib

import mock

from neutron.common import exceptions
from neutron import context
from neutron.db.loadbalancer import loadbalancer_db
from neutron import manager
from neutron.plugins.common import constants
from neutron.services.loadbalancer.drivers.netscaler import ncc_client
from neutron.services.loadbalancer.drivers.netscaler import netscaler_driver
from neutron.tests.unit.db.loadbalancer import test_db_loadbalancer


LBAAS_DRIVER_CLASS = ('neutron.services.loadbalancer.drivers'
                      '.netscaler.netscaler_driver'
                      '.NetScalerPluginDriver')

NCC_CLIENT_CLASS = ('neutron.services.loadbalancer.drivers'
                    '.netscaler.ncc_client'
                    '.NSClient')

LBAAS_PROVIDER_NAME = 'netscaler'
LBAAS_PROVIDER = ('LOADBALANCER:%s:%s:default' %
                  (LBAAS_PROVIDER_NAME, LBAAS_DRIVER_CLASS))

#Test data
TESTVIP_ID = '52ab5d71-6bb2-457f-8414-22a4ba55efec'
TESTPOOL_ID = 'da477c13-24cd-4c9f-8c19-757a61ef3b9d'
TESTMEMBER_ID = '84dea8bc-3416-4fb0-83f9-2ca6e7173bee'
TESTMONITOR_ID = '9b9245a2-0413-4f15-87ef-9a41ef66048c'

TESTVIP_PORT_ID = '327d9662-ade9-4c74-aaf6-c76f145c1180'
TESTPOOL_PORT_ID = '132c1dbb-d3d8-45aa-96e3-71f2ea51651e'
TESTPOOL_SNATIP_ADDRESS = '10.0.0.50'
TESTPOOL_SNAT_PORT = {
    'id': TESTPOOL_PORT_ID,
    'fixed_ips': [{'ip_address': TESTPOOL_SNATIP_ADDRESS}]
}
TESTVIP_IP = '10.0.1.100'
TESTMEMBER_IP = '10.0.0.5'


class TestLoadBalancerPluginBase(test_db_loadbalancer
                                 .LoadBalancerPluginDbTestCase):

    def setUp(self):
        super(TestLoadBalancerPluginBase, self).setUp(
            lbaas_provider=LBAAS_PROVIDER)
        loaded_plugins = manager.NeutronManager().get_service_plugins()
        self.plugin_instance = loaded_plugins[constants.LOADBALANCER]


class TestNetScalerPluginDriver(TestLoadBalancerPluginBase):

    """Unit tests for the NetScaler LBaaS driver module."""

    def setUp(self):
        mock.patch.object(netscaler_driver, 'LOG').start()

        # mock the NSClient class (REST client)
        client_mock_cls = mock.patch(NCC_CLIENT_CLASS).start()

        #mock the REST methods of the NSClient class
        self.client_mock_instance = client_mock_cls.return_value
        self.create_resource_mock = self.client_mock_instance.create_resource
        self.create_resource_mock.side_effect = mock_create_resource_func
        self.update_resource_mock = self.client_mock_instance.update_resource
        self.update_resource_mock.side_effect = mock_update_resource_func
        self.retrieve_resource_mock = (self.client_mock_instance
                                           .retrieve_resource)
        self.retrieve_resource_mock.side_effect = mock_retrieve_resource_func
        self.remove_resource_mock = self.client_mock_instance.remove_resource
        self.remove_resource_mock.side_effect = mock_remove_resource_func
        super(TestNetScalerPluginDriver, self).setUp()
        self.plugin_instance.drivers[LBAAS_PROVIDER_NAME] = (
            netscaler_driver.NetScalerPluginDriver(self.plugin_instance))
        self.driver = self.plugin_instance.drivers[LBAAS_PROVIDER_NAME]
        self.context = context.get_admin_context()

    def test_create_vip(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                testvip = self._build_testvip_contents(subnet['subnet'],
                                                       pool['pool'])
                expectedvip = self._build_expectedvip_contents(
                    testvip,
                    subnet['subnet'])
                # mock the LBaaS plugin update_status().
                self._mock_update_status()
                # reset the create_resource() mock
                self.create_resource_mock.reset_mock()
                # execute the method under test
                self.driver.create_vip(self.context, testvip)
                # First, assert that create_resource was called once
                # with expected params.
                self.create_resource_mock.assert_called_once_with(
                    None,
                    netscaler_driver.VIPS_RESOURCE,
                    netscaler_driver.VIP_RESOURCE,
                    expectedvip)
                #Finally, assert that the vip object is now ACTIVE
                self.mock_update_status_obj.assert_called_once_with(
                    mock.ANY,
                    loadbalancer_db.Vip,
                    expectedvip['id'],
                    constants.ACTIVE)

    def test_create_vip_without_connection(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                testvip = self._build_testvip_contents(subnet['subnet'],
                                                       pool['pool'])
                expectedvip = self._build_expectedvip_contents(
                    testvip,
                    subnet['subnet'])
                errorcode = ncc_client.NCCException.CONNECTION_ERROR
                self.create_resource_mock.side_effect = (
                    ncc_client.NCCException(errorcode))
                # mock the plugin's update_status()
                self._mock_update_status()
                # reset the create_resource() mock
                self.create_resource_mock.reset_mock()
                # execute the method under test.
                self.driver.create_vip(self.context, testvip)
                # First, assert that update_resource was called once
                # with expected params.
                self.create_resource_mock.assert_called_once_with(
                    None,
                    netscaler_driver.VIPS_RESOURCE,
                    netscaler_driver.VIP_RESOURCE,
                    expectedvip)
                #Finally, assert that the vip object is in ERROR state
                self.mock_update_status_obj.assert_called_once_with(
                    mock.ANY,
                    loadbalancer_db.Vip,
                    testvip['id'],
                    constants.ERROR)

    def test_update_vip(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with self.vip(pool=pool, subnet=subnet) as vip:
                    updated_vip = self._build_updated_testvip_contents(
                        vip['vip'],
                        subnet['subnet'],
                        pool['pool'])
                    expectedvip = self._build_updated_expectedvip_contents(
                        updated_vip,
                        subnet['subnet'],
                        pool['pool'])
                    # mock the plugin's update_status()
                    self._mock_update_status()
                    # reset the update_resource() mock
                    self.update_resource_mock.reset_mock()
                    # execute the method under test
                    self.driver.update_vip(self.context, updated_vip,
                                           updated_vip)
                    vip_resource_path = "%s/%s" % (
                        (netscaler_driver.VIPS_RESOURCE,
                         vip['vip']['id']))
                    # First, assert that update_resource was called once
                    # with expected params.
                    (self.update_resource_mock
                         .assert_called_once_with(
                             None,
                             vip_resource_path,
                             netscaler_driver.VIP_RESOURCE,
                             expectedvip))
                    #Finally, assert that the vip object is now ACTIVE
                    self.mock_update_status_obj.assert_called_once_with(
                        mock.ANY,
                        loadbalancer_db.Vip,
                        vip['vip']['id'],
                        constants.ACTIVE)

    def test_delete_vip(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with contextlib.nested(
                    self.vip(pool=pool, subnet=subnet),
                    mock.patch.object(self.driver.plugin, '_delete_db_vip')
                ) as (vip, mock_delete_db_vip):
                    mock_delete_db_vip.return_value = None
                    #reset the remove_resource() mock
                    self.remove_resource_mock.reset_mock()
                    # execute the method under test
                    self.driver.delete_vip(self.context, vip['vip'])
                    vip_resource_path = "%s/%s" % (
                                        (netscaler_driver.VIPS_RESOURCE,
                                         vip['vip']['id']))
                    # Assert that remove_resource() was called once
                    # with expected params.
                    (self.remove_resource_mock
                         .assert_called_once_with(None, vip_resource_path))

    def test_create_pool(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet'),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_ports'),
            mock.patch.object(self.driver.plugin._core_plugin, 'create_port')
        ) as (subnet, mock_get_subnet, mock_get_ports, mock_create_port):
            mock_get_subnet.return_value = subnet['subnet']
            mock_get_ports.return_value = None
            mock_create_port.return_value = TESTPOOL_SNAT_PORT
            testpool = self._build_testpool_contents(subnet['subnet'])
            expectedpool = self._build_expectedpool_contents(testpool,
                                                             subnet['subnet'])
            #reset the create_resource() mock
            self.create_resource_mock.reset_mock()
            # mock the plugin's update_status()
            self._mock_update_status()
            # execute the method under test
            self.driver.create_pool(self.context, testpool)
            # First, assert that create_resource was called once
            # with expected params.
            (self.create_resource_mock
                 .assert_called_once_with(None,
                                          netscaler_driver.POOLS_RESOURCE,
                                          netscaler_driver.POOL_RESOURCE,
                                          expectedpool))
            #Finally, assert that the pool object is now ACTIVE
            self.mock_update_status_obj.assert_called_once_with(
                mock.ANY,
                loadbalancer_db.Pool,
                expectedpool['id'],
                constants.ACTIVE)

    def test_create_pool_with_error(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet'),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_ports'),
            mock.patch.object(self.driver.plugin._core_plugin, 'create_port')
        ) as (subnet, mock_get_subnet, mock_get_ports, mock_create_port):
            mock_get_subnet.return_value = subnet['subnet']
            mock_get_ports.return_value = None
            mock_create_port.return_value = TESTPOOL_SNAT_PORT
            errorcode = ncc_client.NCCException.CONNECTION_ERROR
            self.create_resource_mock.side_effect = (ncc_client
                                                     .NCCException(errorcode))
            testpool = self._build_testpool_contents(subnet['subnet'])
            expectedpool = self._build_expectedpool_contents(testpool,
                                                             subnet['subnet'])
            # mock the plugin's update_status()
            self._mock_update_status()
            #reset the create_resource() mock
            self.create_resource_mock.reset_mock()
            # execute the method under test.
            self.driver.create_pool(self.context, testpool)
            # Also assert that create_resource was called once
            # with expected params.
            (self.create_resource_mock
                 .assert_called_once_with(None,
                                          netscaler_driver.POOLS_RESOURCE,
                                          netscaler_driver.POOL_RESOURCE,
                                          expectedpool))
            #Finally, assert that the pool object is in ERROR state
            self.mock_update_status_obj.assert_called_once_with(
                mock.ANY,
                loadbalancer_db.Pool,
                expectedpool['id'],
                constants.ERROR)

    def test_create_pool_with_snatportcreate_failure(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet'),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_ports'),
            mock.patch.object(self.driver.plugin._core_plugin, 'create_port')
        ) as (subnet, mock_get_subnet, mock_get_ports, mock_create_port):
            mock_get_subnet.return_value = subnet['subnet']
            mock_get_ports.return_value = None
            mock_create_port.side_effect = exceptions.NeutronException()
            testpool = self._build_testpool_contents(subnet['subnet'])
            #reset the create_resource() mock
            self.create_resource_mock.reset_mock()
            # execute the method under test.
            self.assertRaises(exceptions.NeutronException,
                              self.driver.create_pool,
                              self.context, testpool)

    def test_update_pool(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                updated_pool = self._build_updated_testpool_contents(
                    pool['pool'],
                    subnet['subnet'])
                expectedpool = self._build_updated_expectedpool_contents(
                    updated_pool,
                    subnet['subnet'])
                # mock the plugin's update_status()
                self._mock_update_status()
                # reset the update_resource() mock
                self.update_resource_mock.reset_mock()
                # execute the method under test.
                self.driver.update_pool(self.context, pool['pool'],
                                        updated_pool)
                pool_resource_path = "%s/%s" % (
                    (netscaler_driver.POOLS_RESOURCE,
                     pool['pool']['id']))
                # First, assert that update_resource was called once
                # with expected params.
                (self.update_resource_mock
                     .assert_called_once_with(None,
                                              pool_resource_path,
                                              netscaler_driver.POOL_RESOURCE,
                                              expectedpool))
                #Finally, assert that the pool object is now ACTIVE
                self.mock_update_status_obj.assert_called_once_with(
                    mock.ANY,
                    loadbalancer_db.Pool,
                    pool['pool']['id'],
                    constants.ACTIVE)

    def test_delete_pool(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with contextlib.nested(
                self.pool(provider=LBAAS_PROVIDER_NAME),
                mock.patch.object(self.driver.plugin._core_plugin,
                                  'delete_port'),
                mock.patch.object(self.driver.plugin._core_plugin,
                                  'get_ports'),
                mock.patch.object(self.driver.plugin,
                                  'get_pools'),
                mock.patch.object(self.driver.plugin,
                                  '_delete_db_pool')
            ) as (pool, mock_delete_port, mock_get_ports, mock_get_pools,
                  mock_delete_db_pool):
                mock_delete_port.return_value = None
                mock_get_ports.return_value = [{'id': TESTPOOL_PORT_ID}]
                mock_get_pools.return_value = []
                mock_delete_db_pool.return_value = None
                #reset the remove_resource() mock
                self.remove_resource_mock.reset_mock()
                # execute the method under test.
                self.driver.delete_pool(self.context, pool['pool'])
                pool_resource_path = "%s/%s" % (
                    (netscaler_driver.POOLS_RESOURCE,
                     pool['pool']['id']))
                # Assert that delete_resource was called
                # once with expected params.
                (self.remove_resource_mock
                     .assert_called_once_with(None, pool_resource_path))

    def test_create_member(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin,
                              'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                testmember = self._build_testmember_contents(pool['pool'])
                expectedmember = self._build_expectedmember_contents(
                    testmember)
                # mock the plugin's update_status()
                self._mock_update_status()
                #reset the create_resource() mock
                self.create_resource_mock.reset_mock()
                # execute the method under test.
                self.driver.create_member(self.context, testmember)
                # First, assert that create_resource was called once
                # with expected params.
                (self.create_resource_mock
                     .assert_called_once_with(
                         None,
                         netscaler_driver.POOLMEMBERS_RESOURCE,
                         netscaler_driver.POOLMEMBER_RESOURCE,
                         expectedmember))
                #Finally, assert that the member object is now ACTIVE
                self.mock_update_status_obj.assert_called_once_with(
                    mock.ANY,
                    loadbalancer_db.Member,
                    expectedmember['id'],
                    constants.ACTIVE)

    def test_update_member(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with self.member(pool_id=pool['pool']['id']) as member:
                    updatedmember = (self._build_updated_testmember_contents(
                        member['member']))
                    expectedmember = (self
                                      ._build_updated_expectedmember_contents(
                                          updatedmember))
                    # mock the plugin's update_status()
                    self._mock_update_status()
                    # reset the update_resource() mock
                    self.update_resource_mock.reset_mock()
                    # execute the method under test
                    self.driver.update_member(self.context,
                                              member['member'],
                                              updatedmember)
                    member_resource_path = "%s/%s" % (
                        (netscaler_driver.POOLMEMBERS_RESOURCE,
                         member['member']['id']))
                    # First, assert that update_resource was called once
                    # with expected params.
                    (self.update_resource_mock
                         .assert_called_once_with(
                             None,
                             member_resource_path,
                             netscaler_driver.POOLMEMBER_RESOURCE,
                             expectedmember))
                    #Finally, assert that the member object is now ACTIVE
                    self.mock_update_status_obj.assert_called_once_with(
                        mock.ANY,
                        loadbalancer_db.Member,
                        member['member']['id'],
                        constants.ACTIVE)

    def test_delete_member(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with contextlib.nested(
                    self.member(pool_id=pool['pool']['id']),
                    mock.patch.object(self.driver.plugin, '_delete_db_member')
                ) as (member, mock_delete_db_member):
                    mock_delete_db_member.return_value = None
                    # reset the remove_resource() mock
                    self.remove_resource_mock.reset_mock()
                    # execute the method under test
                    self.driver.delete_member(self.context,
                                              member['member'])
                    member_resource_path = "%s/%s" % (
                        (netscaler_driver.POOLMEMBERS_RESOURCE,
                         member['member']['id']))
                    # Assert that delete_resource was called once
                    # with expected params.
                    (self.remove_resource_mock
                         .assert_called_once_with(None, member_resource_path))

    def test_create_pool_health_monitor(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                testhealthmonitor = self._build_testhealthmonitor_contents(
                    pool['pool'])
                expectedhealthmonitor = (
                    self._build_expectedhealthmonitor_contents(
                        testhealthmonitor))
                with mock.patch.object(self.driver.plugin,
                                       'update_pool_health_monitor') as mhm:
                    # reset the create_resource() mock
                    self.create_resource_mock.reset_mock()
                    # execute the method under test.
                    self.driver.create_pool_health_monitor(self.context,
                                                           testhealthmonitor,
                                                           pool['pool']['id'])
                    # First, assert that create_resource was called once
                    # with expected params.
                    resource_path = "%s/%s/%s" % (
                        netscaler_driver.POOLS_RESOURCE,
                        pool['pool']['id'],
                        netscaler_driver.MONITORS_RESOURCE)
                    (self.create_resource_mock
                         .assert_called_once_with(
                             None,
                             resource_path,
                             netscaler_driver.MONITOR_RESOURCE,
                             expectedhealthmonitor))
                    # Finally, assert that the healthmonitor object is
                    # now ACTIVE.
                    (mhm.assert_called_once_with(
                        mock.ANY,
                        expectedhealthmonitor['id'],
                        pool['pool']['id'],
                        constants.ACTIVE, ""))

    def test_update_pool_health_monitor(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with self.health_monitor(
                    pool_id=pool['pool']['id']
                ) as (health_monitor):
                    updatedhealthmonitor = (
                        self._build_updated_testhealthmonitor_contents(
                            health_monitor['health_monitor']))
                    expectedhealthmonitor = (
                        self._build_updated_expectedhealthmonitor_contents(
                            updatedhealthmonitor))
                    with mock.patch.object(self.driver.plugin,
                                           'update_pool_health_monitor')as mhm:
                        # reset the update_resource() mock
                        self.update_resource_mock.reset_mock()
                        # execute the method under test.
                        self.driver.update_pool_health_monitor(
                            self.context,
                            health_monitor['health_monitor'],
                            updatedhealthmonitor,
                            pool['pool']['id'])
                        monitor_resource_path = "%s/%s" % (
                            (netscaler_driver.MONITORS_RESOURCE,
                             health_monitor['health_monitor']['id']))
                        # First, assert that update_resource was called once
                        # with expected params.
                        self.update_resource_mock.assert_called_once_with(
                            None,
                            monitor_resource_path,
                            netscaler_driver.MONITOR_RESOURCE,
                            expectedhealthmonitor)
                        #Finally, assert that the member object is now ACTIVE
                        (mhm.assert_called_once_with(
                            mock.ANY,
                            health_monitor['health_monitor']['id'],
                            pool['pool']['id'],
                            constants.ACTIVE, ""))

    def test_delete_pool_health_monitor(self):
        with contextlib.nested(
            self.subnet(),
            mock.patch.object(self.driver.plugin._core_plugin, 'get_subnet')
        ) as (subnet, mock_get_subnet):
            mock_get_subnet.return_value = subnet['subnet']
            with self.pool(provider=LBAAS_PROVIDER_NAME) as pool:
                with contextlib.nested(
                    self.health_monitor(pool_id=pool['pool']['id']),
                    mock.patch.object(self.driver.plugin,
                                      '_delete_db_pool_health_monitor')
                ) as (health_monitor, mock_delete_db_monitor):
                    mock_delete_db_monitor.return_value = None
                    # reset the remove_resource() mock
                    self.remove_resource_mock.reset_mock()
                    # execute the method under test.
                    self.driver.delete_pool_health_monitor(
                        self.context,
                        health_monitor['health_monitor'],
                        pool['pool']['id'])
                    monitor_resource_path = "%s/%s/%s/%s" % (
                        netscaler_driver.POOLS_RESOURCE,
                        pool['pool']['id'],
                        netscaler_driver.MONITORS_RESOURCE,
                        health_monitor['health_monitor']['id'])
                    # Assert that delete_resource was called once
                    # with expected params.
                    self.remove_resource_mock.assert_called_once_with(
                        None,
                        monitor_resource_path)

    def _build_testvip_contents(self, subnet, pool):
        vip_obj = dict(id=TESTVIP_ID,
                       name='testvip',
                       description='a test vip',
                       tenant_id=self._tenant_id,
                       subnet_id=subnet['id'],
                       address=TESTVIP_IP,
                       port_id=TESTVIP_PORT_ID,
                       pool_id=pool['id'],
                       protocol='HTTP',
                       protocol_port=80,
                       connection_limit=1000,
                       admin_state_up=True,
                       status='PENDING_CREATE',
                       status_description='')
        return vip_obj

    def _build_expectedvip_contents(self, testvip, subnet):
        expectedvip = dict(id=testvip['id'],
                           name=testvip['name'],
                           description=testvip['description'],
                           tenant_id=testvip['tenant_id'],
                           subnet_id=testvip['subnet_id'],
                           address=testvip['address'],
                           network_id=subnet['network_id'],
                           port_id=testvip['port_id'],
                           pool_id=testvip['pool_id'],
                           protocol=testvip['protocol'],
                           protocol_port=testvip['protocol_port'],
                           connection_limit=testvip['connection_limit'],
                           admin_state_up=testvip['admin_state_up'])
        return expectedvip

    def _build_updated_testvip_contents(self, testvip, subnet, pool):
        #update some updateable fields of the vip
        testvip['name'] = 'udpated testvip'
        testvip['description'] = 'An updated version of test vip'
        testvip['connection_limit'] = 2000
        return testvip

    def _build_updated_expectedvip_contents(self, testvip, subnet, pool):
        expectedvip = dict(name=testvip['name'],
                           description=testvip['description'],
                           connection_limit=testvip['connection_limit'],
                           admin_state_up=testvip['admin_state_up'],
                           pool_id=testvip['pool_id'])
        return expectedvip

    def _build_testpool_contents(self, subnet):
        pool_obj = dict(id=TESTPOOL_ID,
                        name='testpool',
                        description='a test pool',
                        tenant_id=self._tenant_id,
                        subnet_id=subnet['id'],
                        protocol='HTTP',
                        vip_id=None,
                        admin_state_up=True,
                        lb_method='ROUND_ROBIN',
                        status='PENDING_CREATE',
                        status_description='',
                        members=[],
                        health_monitors=[],
                        health_monitors_status=None,
                        provider=LBAAS_PROVIDER_NAME)
        return pool_obj

    def _build_expectedpool_contents(self, testpool, subnet):
        expectedpool = dict(id=testpool['id'],
                            name=testpool['name'],
                            description=testpool['description'],
                            tenant_id=testpool['tenant_id'],
                            subnet_id=testpool['subnet_id'],
                            network_id=subnet['network_id'],
                            protocol=testpool['protocol'],
                            vip_id=testpool['vip_id'],
                            lb_method=testpool['lb_method'],
                            snat_ip=TESTPOOL_SNATIP_ADDRESS,
                            port_id=TESTPOOL_PORT_ID,
                            admin_state_up=testpool['admin_state_up'])
        return expectedpool

    def _build_updated_testpool_contents(self, testpool, subnet):
        updated_pool = dict(testpool.items())
        updated_pool['name'] = 'udpated testpool'
        updated_pool['description'] = 'An updated version of test pool'
        updated_pool['lb_method'] = 'LEAST_CONNECTIONS'
        updated_pool['admin_state_up'] = True
        updated_pool['provider'] = LBAAS_PROVIDER_NAME
        updated_pool['status'] = 'PENDING_UPDATE'
        updated_pool['status_description'] = ''
        updated_pool['members'] = []
        updated_pool["health_monitors"] = []
        updated_pool["health_monitors_status"] = None
        return updated_pool

    def _build_updated_expectedpool_contents(self, testpool, subnet):
        expectedpool = dict(name=testpool['name'],
                            description=testpool['description'],
                            lb_method=testpool['lb_method'],
                            admin_state_up=testpool['admin_state_up'])
        return expectedpool

    def _build_testmember_contents(self, pool):
        member_obj = dict(
            id=TESTMEMBER_ID,
            tenant_id=self._tenant_id,
            pool_id=pool['id'],
            address=TESTMEMBER_IP,
            protocol_port=8080,
            weight=2,
            admin_state_up=True,
            status='PENDING_CREATE',
            status_description='')
        return member_obj

    def _build_expectedmember_contents(self, testmember):
        expectedmember = dict(
            id=testmember['id'],
            tenant_id=testmember['tenant_id'],
            pool_id=testmember['pool_id'],
            address=testmember['address'],
            protocol_port=testmember['protocol_port'],
            weight=testmember['weight'],
            admin_state_up=testmember['admin_state_up'])
        return expectedmember

    def _build_updated_testmember_contents(self, testmember):
        updated_member = dict(testmember.items())
        updated_member.update(
            weight=3,
            admin_state_up=True,
            status='PENDING_CREATE',
            status_description=''
        )
        return updated_member

    def _build_updated_expectedmember_contents(self, testmember):
        expectedmember = dict(weight=testmember['weight'],
                              pool_id=testmember['pool_id'],
                              admin_state_up=testmember['admin_state_up'])
        return expectedmember

    def _build_testhealthmonitor_contents(self, pool):
        monitor_obj = dict(
            id=TESTMONITOR_ID,
            tenant_id=self._tenant_id,
            type='TCP',
            delay=10,
            timeout=5,
            max_retries=3,
            admin_state_up=True,
            pools=[])
        pool_obj = dict(status='PENDING_CREATE',
                        status_description=None,
                        pool_id=pool['id'])
        monitor_obj['pools'].append(pool_obj)
        return monitor_obj

    def _build_expectedhealthmonitor_contents(self, testhealthmonitor):
        expectedmonitor = dict(id=testhealthmonitor['id'],
                               tenant_id=testhealthmonitor['tenant_id'],
                               type=testhealthmonitor['type'],
                               delay=testhealthmonitor['delay'],
                               timeout=testhealthmonitor['timeout'],
                               max_retries=testhealthmonitor['max_retries'],
                               admin_state_up=(
                                   testhealthmonitor['admin_state_up']))
        return expectedmonitor

    def _build_updated_testhealthmonitor_contents(self, testmonitor):
        updated_monitor = dict(testmonitor.items())
        updated_monitor.update(
            delay=30,
            timeout=3,
            max_retries=5,
            admin_state_up=True
        )
        return updated_monitor

    def _build_updated_expectedhealthmonitor_contents(self, testmonitor):
        expectedmonitor = dict(delay=testmonitor['delay'],
                               timeout=testmonitor['timeout'],
                               max_retries=testmonitor['max_retries'],
                               admin_state_up=testmonitor['admin_state_up'])
        return expectedmonitor

    def _mock_update_status(self):
        #patch the plugin's update_status() method with a mock object
        self.mock_update_status_patcher = mock.patch.object(
            self.driver.plugin,
            'update_status')
        self.mock_update_status_obj = self.mock_update_status_patcher.start()


def mock_create_resource_func(*args, **kwargs):
    return 201, {}


def mock_update_resource_func(*args, **kwargs):
    return 202, {}


def mock_retrieve_resource_func(*args, **kwargs):
    return 200, {}


def mock_remove_resource_func(*args, **kwargs):
    return 200, {}



# Copyright 2014 Cisco Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import mock

from neutron.common import config
from neutron import context
from neutron.plugins.cisco.common import cisco_constants as const
from neutron.plugins.cisco.common import config as cisco_config
from neutron.plugins.cisco.models import virt_phy_sw_v2
from neutron.plugins.cisco.nexus import cisco_nexus_plugin_v2
from neutron.tests import base
from neutron.tests.unit import test_api_v2


class TestCiscoPluginModel(base.BaseTestCase):

    def setUp(self):
        # Point config file to: neutron/tests/etc/neutron.conf.test
        args = ['--config-file', test_api_v2.etcdir('neutron.conf.test')]
        config.parse(args=args)

        super(TestCiscoPluginModel, self).setUp()

    def test_non_nexus_device_driver(self):
        """Tests handling of an non-Nexus device driver being configured."""
        with mock.patch.dict(sys.modules, {'mock_driver': mock.Mock()}):
            cisco_config.CONF.set_override('nexus_driver',
                                           'mock_driver.Non_Nexus_Driver',
                                           'CISCO')
            # Plugin model instance should have is_nexus_plugin set to False
            model = virt_phy_sw_v2.VirtualPhysicalSwitchModelV2()
            self.assertFalse(model.is_nexus_plugin)

            # Model's _invoke_nexus_for_net_create should just return False
            user_id = 'user_id'
            tenant_id = 'tenant_id'
            ctx = context.Context(user_id, tenant_id)
            self.assertFalse(model._invoke_nexus_for_net_create(
                ctx, tenant_id, net_id='net_id',
                instance_id='instance_id', host_id='host_id'))

    def test_nexus_plugin_calls_ignored_if_plugin_not_loaded(self):
        """Verifies Nexus plugin calls are ignored if plugin is not loaded."""
        cisco_config.CONF.set_override(const.NEXUS_PLUGIN,
                                       None, 'CISCO_PLUGINS')
        with mock.patch.object(cisco_nexus_plugin_v2.NexusPlugin,
                               'create_network') as mock_create_network:
            model = virt_phy_sw_v2.VirtualPhysicalSwitchModelV2()
            model._invoke_plugin_per_device(model, const.NEXUS_PLUGIN,
                                            'create_network')
            self.assertFalse(mock_create_network.called)



# Copyright 2013 VMware, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64

import eventlet

from neutron.openstack.common import jsonutils
from neutron.plugins.vmware.vshield.common import exceptions

httplib2 = eventlet.import_patched('httplib2')


def xmldumps(obj):
    config = ""
    if isinstance(obj, dict):
        for key, value in obj.iteritems():
            cfg = "<%s>%s</%s>" % (key, xmldumps(value), key)
            config += cfg
    elif isinstance(obj, list):
        for value in obj:
            config += xmldumps(value)
    else:
        config = obj

    return config


class VcnsApiHelper(object):
    errors = {
        303: exceptions.ResourceRedirect,
        400: exceptions.RequestBad,
        403: exceptions.Forbidden,
        404: exceptions.ResourceNotFound,
        415: exceptions.MediaTypeUnsupport,
        503: exceptions.ServiceUnavailable
    }

    def __init__(self, address, user, password, format='json'):
        self.authToken = base64.encodestring("%s:%s" % (user, password))
        self.user = user
        self.passwd = password
        self.address = address
        self.format = format
        if format == 'json':
            self.encode = jsonutils.dumps
        else:
            self.encode = xmldumps

    def request(self, method, uri, params=None):
        uri = self.address + uri
        http = httplib2.Http()
        http.disable_ssl_certificate_validation = True
        headers = {
            'Content-Type': 'application/' + self.format,
            'Accept': 'application/' + 'json',
            'Authorization': 'Basic ' + self.authToken
        }
        body = self.encode(params) if params else None
        header, response = http.request(uri, method,
                                        body=body, headers=headers)
        status = int(header['status'])
        if 200 <= status < 300:
            return header, response
        if status in self.errors:
            cls = self.errors[status]
        else:
            cls = exceptions.VcnsApiException
        raise cls(uri=uri, status=status, header=header, response=response)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Ying Liu, Cisco Systems, Inc.
# @author: Abhishek Raut, Cisco Systems, Inc

from neutron.api import extensions
from neutron.api.v2 import attributes
from neutron.api.v2 import base
from neutron import manager


# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    'credentials': {
        'credential_id': {'allow_post': False, 'allow_put': False,
                          'validate': {'type:regex': attributes.UUID_PATTERN},
                          'is_visible': True},
        'credential_name': {'allow_post': True, 'allow_put': True,
                            'is_visible': True, 'default': ''},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'is_visible': False, 'default': ''},
        'type': {'allow_post': True, 'allow_put': True,
                 'is_visible': True, 'default': ''},
        'user_name': {'allow_post': True, 'allow_put': True,
                      'is_visible': True, 'default': ''},
        'password': {'allow_post': True, 'allow_put': True,
                     'is_visible': True, 'default': ''},
    },
}


class Credential(extensions.ExtensionDescriptor):

    @classmethod
    def get_name(cls):
        """Returns Extended Resource Name."""
        return "Cisco Credential"

    @classmethod
    def get_alias(cls):
        """Returns Extended Resource Alias."""
        return "credential"

    @classmethod
    def get_description(cls):
        """Returns Extended Resource Description."""
        return "Credential include username and password"

    @classmethod
    def get_namespace(cls):
        """Returns Extended Resource Namespace."""
        return "http://docs.ciscocloud.com/api/ext/credential/v2.0"

    @classmethod
    def get_updated(cls):
        """Returns Extended Resource Update Time."""
        return "2011-07-25T13:25:27-06:00"

    @classmethod
    def get_resources(cls):
        """Returns Extended Resources."""
        resource_name = "credential"
        collection_name = resource_name + "s"
        plugin = manager.NeutronManager.get_plugin()
        params = RESOURCE_ATTRIBUTE_MAP.get(collection_name, dict())
        controller = base.create_resource(collection_name,
                                          resource_name,
                                          plugin, params)
        return [extensions.ResourceExtension(collection_name,
                                             controller)]



# Copyright 2011 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Routines for configuring Neutron
"""

import os

from oslo.config import cfg
from paste import deploy

from neutron.api.v2 import attributes
from neutron.common import utils
from neutron.openstack.common.db.sqlalchemy import session as db_session
from neutron.openstack.common import log as logging
from neutron.openstack.common import rpc
from neutron.version import version_info as neutron_version


LOG = logging.getLogger(__name__)

core_opts = [
    cfg.StrOpt('bind_host', default='0.0.0.0',
               help=_("The host IP to bind to")),
    cfg.IntOpt('bind_port', default=9696,
               help=_("The port to bind to")),
    cfg.StrOpt('api_paste_config', default="api-paste.ini",
               help=_("The API paste config file to use")),
    cfg.StrOpt('api_extensions_path', default="",
               help=_("The path for API extensions")),
    cfg.StrOpt('policy_file', default="policy.json",
               help=_("The policy file to use")),
    cfg.StrOpt('auth_strategy', default='keystone',
               help=_("The type of authentication to use")),
    cfg.StrOpt('core_plugin',
               help=_("The core plugin Neutron will use")),
    cfg.ListOpt('service_plugins', default=[],
                help=_("The service plugins Neutron will use")),
    cfg.StrOpt('base_mac', default="fa:16:3e:00:00:00",
               help=_("The base MAC address Neutron will use for VIFs")),
    cfg.IntOpt('mac_generation_retries', default=16,
               help=_("How many times Neutron will retry MAC generation")),
    cfg.BoolOpt('allow_bulk', default=True,
                help=_("Allow the usage of the bulk API")),
    cfg.BoolOpt('allow_pagination', default=False,
                help=_("Allow the usage of the pagination")),
    cfg.BoolOpt('allow_sorting', default=False,
                help=_("Allow the usage of the sorting")),
    cfg.StrOpt('pagination_max_limit', default="-1",
               help=_("The maximum number of items returned in a single "
                      "response, value was 'infinite' or negative integer "
                      "means no limit")),
    cfg.IntOpt('max_dns_nameservers', default=5,
               help=_("Maximum number of DNS nameservers")),
    cfg.IntOpt('max_subnet_host_routes', default=20,
               help=_("Maximum number of host routes per subnet")),
    cfg.IntOpt('max_fixed_ips_per_port', default=5,
               help=_("Maximum number of fixed ips per port")),
    cfg.IntOpt('dhcp_lease_duration', default=86400,
               deprecated_name='dhcp_lease_time',
               help=_("DHCP lease duration")),
    cfg.BoolOpt('dhcp_agent_notification', default=True,
                help=_("Allow sending resource operation"
                       " notification to DHCP agent")),
    cfg.BoolOpt('allow_overlapping_ips', default=False,
                help=_("Allow overlapping IP support in Neutron")),
    cfg.StrOpt('host', default=utils.get_hostname(),
               help=_("The hostname Neutron is running on")),
    cfg.BoolOpt('force_gateway_on_subnet', default=False,
                help=_("Ensure that configured gateway is on subnet")),
    cfg.BoolOpt('notify_nova_on_port_status_changes', default=True,
                help=_("Send notification to nova when port status changes")),
    cfg.BoolOpt('notify_nova_on_port_data_changes', default=True,
                help=_("Send notification to nova when port data (fixed_ips/"
                       "floatingip) changes so nova can update its cache.")),
    cfg.StrOpt('nova_url',
               default='http://127.0.0.1:8774',
               help=_('URL for connection to nova')),
    cfg.StrOpt('nova_admin_username',
               help=_('Username for connecting to nova in admin context')),
    cfg.StrOpt('nova_admin_password',
               help=_('Password for connection to nova in admin context'),
               secret=True),
    cfg.StrOpt('nova_admin_tenant_id',
               help=_('The uuid of the admin nova tenant')),
    cfg.StrOpt('nova_admin_auth_url',
               default='http://localhost:5000/v2.0',
               help=_('Authorization URL for connecting to nova in admin '
                      'context')),
    cfg.StrOpt('nova_region_name',
               help=_('Name of nova region to use. Useful if keystone manages'
                      ' more than one region.')),
    cfg.IntOpt('send_events_interval', default=2,
               help=_('Number of seconds between sending events to nova if '
                      'there are any events to send.')),
]

core_cli_opts = [
    cfg.StrOpt('state_path',
               default='/var/lib/neutron',
               help=_("Where to store Neutron state files. "
                      "This directory must be writable by the agent.")),
]

# Register the configuration options
cfg.CONF.register_opts(core_opts)
cfg.CONF.register_cli_opts(core_cli_opts)

# Ensure that the control exchange is set correctly
rpc.set_defaults(control_exchange='neutron')
_SQL_CONNECTION_DEFAULT = 'sqlite://'
# Update the default QueuePool parameters. These can be tweaked by the
# configuration variables - max_pool_size, max_overflow and pool_timeout
db_session.set_defaults(sql_connection=_SQL_CONNECTION_DEFAULT,
                        sqlite_db='', max_pool_size=10,
                        max_overflow=20, pool_timeout=10)


def parse(args):
    cfg.CONF(args=args, project='neutron',
             version='%%prog %s' % neutron_version.release_string())

    # Validate that the base_mac is of the correct format
    msg = attributes._validate_regex(cfg.CONF.base_mac,
                                     attributes.MAC_PATTERN)
    if msg:
        msg = _("Base MAC: %s") % msg
        raise Exception(msg)


def setup_logging(conf):
    """Sets up the logging options for a log with supplied name.

    :param conf: a cfg.ConfOpts object
    """
    product_name = "neutron"
    logging.setup(product_name)
    LOG.info(_("Logging enabled!"))


def load_paste_app(app_name):
    """Builds and returns a WSGI app from a paste config file.

    :param app_name: Name of the application to load
    :raises ConfigFilesNotFoundError when config file cannot be located
    :raises RuntimeError when application cannot be loaded from config file
    """

    config_path = cfg.CONF.find_file(cfg.CONF.api_paste_config)
    if not config_path:
        raise cfg.ConfigFilesNotFoundError(
            config_files=[cfg.CONF.api_paste_config])
    config_path = os.path.abspath(config_path)
    LOG.info(_("Config paste file: %s"), config_path)

    try:
        app = deploy.loadapp("config:%s" % config_path, name=app_name)
    except (LookupError, ImportError):
        msg = (_("Unable to load %(app_name)s from "
                 "configuration file %(config_path)s.") %
               {'app_name': app_name,
                'config_path': config_path})
        LOG.exception(msg)
        raise RuntimeError(msg)
    return app



# Copyright 2014 OneConvergence, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock

from neutron.openstack.common import jsonutils as json
from neutron.plugins.oneconvergence.lib import nvsdlib
from neutron.tests import base

NETWORKS_URI = "/pluginhandler/ocplugin/tenant/%s/lnetwork/"
NETWORK_URI = NETWORKS_URI + "%s"
GET_ALL_NETWORKS = "/pluginhandler/ocplugin/tenant/getallnetworks"

SUBNETS_URI = NETWORK_URI + "/lsubnet/"
SUBNET_URI = SUBNETS_URI + "%s"
GET_ALL_SUBNETS = "/pluginhandler/ocplugin/tenant/getallsubnets"

PORTS_URI = NETWORK_URI + "/lport/"
PORT_URI = PORTS_URI + "%s"

EXT_URI = "/pluginhandler/ocplugin/ext/tenant/%s"
FLOATING_IPS_URI = EXT_URI + "/floatingip/"
FLOATING_IP_URI = FLOATING_IPS_URI + "%s"

ROUTERS_URI = EXT_URI + "/lrouter/"
ROUTER_URI = ROUTERS_URI + "%s"

TEST_NET = 'test-network'
TEST_SUBNET = 'test-subnet'
TEST_PORT = 'test-port'
TEST_FIP = 'test-floatingip'
TEST_ROUTER = 'test-router'
TEST_TENANT = 'test-tenant'


class TestNVSDApi(base.BaseTestCase):

    def setUp(self):
        super(TestNVSDApi, self).setUp()
        self.nvsdlib = nvsdlib.NVSDApi()

    def test_create_network(self):
        network_obj = {
            "name": 'test-net',
            "tenant_id": TEST_TENANT,
            "shared": False,
            "admin_state_up": True,
            "router:external": False
        }
        resp = mock.Mock()
        resp.json.return_value = {'id': 'uuid'}
        with mock.patch.object(self.nvsdlib, 'send_request',
                               return_value=resp) as send_request:
            uri = NETWORKS_URI % TEST_TENANT
            net = self.nvsdlib.create_network(network_obj)
            send_request.assert_called_once_with("POST", uri,
                                                 body=json.dumps(network_obj),
                                                 resource='network',
                                                 tenant_id=TEST_TENANT)
            self.assertEqual(net, {'id': 'uuid'})

    def test_update_network(self):
        network = {'id': TEST_NET,
                   'tenant_id': TEST_TENANT}
        update_network = {'name': 'new_name'}
        uri = NETWORK_URI % (TEST_TENANT, TEST_NET)
        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.update_network(network, update_network)
            send_request.assert_called_once_with(
                "PUT", uri, body=json.dumps(update_network),
                resource='network', tenant_id=TEST_TENANT,
                resource_id=TEST_NET)

    def test_delete_network(self):
        network = {'id': TEST_NET,
                   'tenant_id': TEST_TENANT}

        uri = NETWORK_URI % (TEST_TENANT, TEST_NET)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            with mock.patch.object(self.nvsdlib, '_get_ports'):
                self.nvsdlib.delete_network(network)
                send_request.assert_called_once_with(
                    "DELETE", uri, resource='network',
                    tenant_id=TEST_TENANT, resource_id=TEST_NET)

    def test_create_port(self):
        path = PORTS_URI % (TEST_TENANT, TEST_NET)
        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            fixed_ips = [{'ip_address': '10.0.0.2',
                          'subnet_id': TEST_SUBNET}]

            lport = {
                "id": TEST_PORT,
                "name": 'test',
                "device_id": "device_id",
                "device_owner": "device_owner",
                "mac_address": "mac_address",
                "fixed_ips": fixed_ips,
                "admin_state_up": True,
                "network_id": TEST_NET,
                "status": 'ACTIVE'
            }
            self.nvsdlib.create_port(TEST_TENANT, lport)
            expected = {"id": TEST_PORT, "name": 'test',
                        "device_id": "device_id",
                        "device_owner": "device_owner",
                        "mac_address": "mac_address",
                        "ip_address": '10.0.0.2',
                        "subnet_id": TEST_SUBNET,
                        "admin_state_up": True,
                        "network_id": TEST_NET,
                        "status": 'ACTIVE'}
            send_request.assert_called_once_with("POST", path,
                                                 body=json.dumps(expected),
                                                 resource='port',
                                                 tenant_id=TEST_TENANT)

    def test_update_port(self):
        port = {'id': TEST_PORT,
                'network_id': TEST_NET}

        port_update = {'name': 'new-name'}
        uri = PORT_URI % (TEST_TENANT, TEST_NET, TEST_PORT)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.update_port(TEST_TENANT, port, port_update)
            send_request.assert_called_once_with("PUT", uri,
                                                 body=json.dumps(port_update),
                                                 resource='port',
                                                 resource_id='test-port',
                                                 tenant_id=TEST_TENANT)

    def test_delete_port(self):
        port = {'network_id': TEST_NET,
                'tenant_id': TEST_TENANT}
        uri = PORT_URI % (TEST_TENANT, TEST_NET, TEST_PORT)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.delete_port(TEST_PORT, port)
            send_request.assert_called_once_with("DELETE", uri,
                                                 resource='port',
                                                 tenant_id=TEST_TENANT,
                                                 resource_id=TEST_PORT)

    def test_create_subnet(self):
        subnet = {'id': TEST_SUBNET,
                  'tenant_id': TEST_TENANT,
                  'network_id': TEST_NET}
        uri = SUBNETS_URI % (TEST_TENANT, TEST_NET)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.create_subnet(subnet)
            send_request.assert_called_once_with("POST", uri,
                                                 body=json.dumps(subnet),
                                                 resource='subnet',
                                                 tenant_id=TEST_TENANT)

    def test_update_subnet(self):
        subnet = {'id': TEST_SUBNET,
                  'tenant_id': TEST_TENANT,
                  'network_id': TEST_NET}
        subnet_update = {'name': 'new-name'}
        uri = SUBNET_URI % (TEST_TENANT, TEST_NET, TEST_SUBNET)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.update_subnet(subnet, subnet_update)
            send_request.assert_called_once_with(
                "PUT", uri, body=json.dumps(subnet_update), resource='subnet',
                tenant_id=TEST_TENANT, resource_id=TEST_SUBNET)

    def test_delete_subnet(self):
        subnet = {'id': TEST_SUBNET,
                  'tenant_id': TEST_TENANT,
                  'network_id': TEST_NET}
        uri = SUBNET_URI % (TEST_TENANT, TEST_NET, TEST_SUBNET)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.delete_subnet(subnet)
            send_request.assert_called_once_with("DELETE", uri,
                                                 resource='subnet',
                                                 tenant_id=TEST_TENANT,
                                                 resource_id=TEST_SUBNET)

    def test_create_floatingip(self):
        floatingip = {'id': TEST_FIP,
                      'tenant_id': TEST_TENANT}
        uri = FLOATING_IPS_URI % TEST_TENANT

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.create_floatingip(floatingip)
            send_request.assert_called_once_with("POST", uri,
                                                 body=json.dumps(floatingip),
                                                 resource='floating_ip',
                                                 tenant_id=TEST_TENANT)

    def test_update_floatingip(self):
        floatingip = {'id': TEST_FIP,
                      'tenant_id': TEST_TENANT}
        uri = FLOATING_IP_URI % (TEST_TENANT, TEST_FIP)

        floatingip_update = {'floatingip': {'router_id': TEST_ROUTER}}
        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.update_floatingip(floatingip, floatingip_update)
            send_request.assert_called_once_with(
                "PUT", uri, body=json.dumps(floatingip_update['floatingip']),
                resource='floating_ip', tenant_id=TEST_TENANT,
                resource_id=TEST_FIP)

    def test_delete_floatingip(self):
        floatingip = {'id': TEST_FIP,
                      'tenant_id': TEST_TENANT}
        uri = FLOATING_IP_URI % (TEST_TENANT, TEST_FIP)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.delete_floatingip(floatingip)
            send_request.assert_called_once_with(
                "DELETE", uri, resource='floating_ip', tenant_id=TEST_TENANT,
                resource_id=TEST_FIP)

    def test_create_router(self):
        router = {'id': TEST_ROUTER, 'tenant_id': TEST_TENANT}
        uri = ROUTERS_URI % TEST_TENANT

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.create_router(router)
            send_request.assert_called_once_with(
                "POST", uri, body=json.dumps(router), resource='router',
                tenant_id=TEST_TENANT)

    def test_update_router(self):
        router = {'id': TEST_ROUTER, 'tenant_id': TEST_TENANT}
        uri = ROUTER_URI % (TEST_TENANT, TEST_ROUTER)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.update_router(router)
            send_request.assert_called_once_with(
                "PUT", uri, body=json.dumps(router),
                resource='router', tenant_id=TEST_TENANT,
                resource_id=TEST_ROUTER)

    def test_delete_router(self):
        uri = ROUTER_URI % (TEST_TENANT, TEST_ROUTER)

        with mock.patch.object(self.nvsdlib, 'send_request') as send_request:
            self.nvsdlib.delete_router(TEST_TENANT, TEST_ROUTER)
            send_request.assert_called_once_with(
                "DELETE", uri, resource='router',
                tenant_id=TEST_TENANT, resource_id=TEST_ROUTER)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Cisco Systems, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Abhishek Raut, Cisco Systems Inc.
# @author: Rudrajit Tapadar, Cisco Systems Inc.

import sqlalchemy as sa

from neutron.db import model_base
from neutron.db import models_v2
from neutron.openstack.common import log as logging
from neutron.plugins.cisco.common import cisco_constants


LOG = logging.getLogger(__name__)


class N1kvVlanAllocation(model_base.BASEV2):

    """Represents allocation state of vlan_id on physical network."""
    __tablename__ = 'cisco_n1kv_vlan_allocations'

    physical_network = sa.Column(sa.String(64),
                                 nullable=False,
                                 primary_key=True)
    vlan_id = sa.Column(sa.Integer, nullable=False, primary_key=True,
                        autoincrement=False)
    allocated = sa.Column(sa.Boolean, nullable=False, default=False)
    network_profile_id = sa.Column(sa.String(36),
                                   sa.ForeignKey('cisco_network_profiles.id',
                                                 ondelete="CASCADE"),
                                   nullable=False)


class N1kvVxlanAllocation(model_base.BASEV2):

    """Represents allocation state of vxlan_id."""
    __tablename__ = 'cisco_n1kv_vxlan_allocations'

    vxlan_id = sa.Column(sa.Integer, nullable=False, primary_key=True,
                         autoincrement=False)
    allocated = sa.Column(sa.Boolean, nullable=False, default=False)
    network_profile_id = sa.Column(sa.String(36),
                                   sa.ForeignKey('cisco_network_profiles.id',
                                                 ondelete="CASCADE"),
                                   nullable=False)


class N1kvPortBinding(model_base.BASEV2):

    """Represents binding of ports to policy profile."""
    __tablename__ = 'cisco_n1kv_port_bindings'

    port_id = sa.Column(sa.String(36),
                        sa.ForeignKey('ports.id', ondelete="CASCADE"),
                        primary_key=True)
    profile_id = sa.Column(sa.String(36),
                           sa.ForeignKey('cisco_policy_profiles.id'))


class N1kvNetworkBinding(model_base.BASEV2):

    """Represents binding of virtual network to physical realization."""
    __tablename__ = 'cisco_n1kv_network_bindings'

    network_id = sa.Column(sa.String(36),
                           sa.ForeignKey('networks.id', ondelete="CASCADE"),
                           primary_key=True)
    network_type = sa.Column(sa.String(32), nullable=False)
    physical_network = sa.Column(sa.String(64))
    segmentation_id = sa.Column(sa.Integer)
    multicast_ip = sa.Column(sa.String(32))
    profile_id = sa.Column(sa.String(36),
                           sa.ForeignKey('cisco_network_profiles.id'))


class N1kVmNetwork(model_base.BASEV2):

    """Represents VM Network information."""
    __tablename__ = 'cisco_n1kv_vmnetworks'

    name = sa.Column(sa.String(80), primary_key=True)
    profile_id = sa.Column(sa.String(36),
                           sa.ForeignKey('cisco_policy_profiles.id'))
    network_id = sa.Column(sa.String(36))
    port_count = sa.Column(sa.Integer)


class NetworkProfile(model_base.BASEV2, models_v2.HasId):

    """
    Nexus1000V Network Profiles

        segment_type - VLAN, OVERLAY, TRUNK, MULTI_SEGMENT
        sub_type - TRUNK_VLAN, TRUNK_VXLAN, native_vxlan, enhanced_vxlan
        segment_range - '<integer>-<integer>'
        multicast_ip_index - <integer>
        multicast_ip_range - '<ip>-<ip>'
        physical_network - Name for the physical network
    """
    __tablename__ = 'cisco_network_profiles'

    name = sa.Column(sa.String(255))
    segment_type = sa.Column(sa.Enum(cisco_constants.NETWORK_TYPE_VLAN,
                                     cisco_constants.NETWORK_TYPE_OVERLAY,
                                     cisco_constants.NETWORK_TYPE_TRUNK,
                                     cisco_constants.
                                     NETWORK_TYPE_MULTI_SEGMENT,
                                     name='segment_type'),
                             nullable=False)
    sub_type = sa.Column(sa.String(255))
    segment_range = sa.Column(sa.String(255))
    multicast_ip_index = sa.Column(sa.Integer, default=0)
    multicast_ip_range = sa.Column(sa.String(255))
    physical_network = sa.Column(sa.String(255))


class PolicyProfile(model_base.BASEV2):

    """
    Nexus1000V Network Profiles

        Both 'id' and 'name' are coming from Nexus1000V switch
    """
    __tablename__ = 'cisco_policy_profiles'

    id = sa.Column(sa.String(36), primary_key=True)
    name = sa.Column(sa.String(255))


class ProfileBinding(model_base.BASEV2):

    """
    Represents a binding of Network Profile
    or Policy Profile to tenant_id
    """
    __tablename__ = 'cisco_n1kv_profile_bindings'

    profile_type = sa.Column(sa.Enum(cisco_constants.NETWORK,
                                     cisco_constants.POLICY,
                                     name='profile_type'))
    tenant_id = sa.Column(sa.String(36),
                          primary_key=True,
                          default=cisco_constants.TENANT_ID_NOT_SET)
    profile_id = sa.Column(sa.String(36), primary_key=True)


class N1kvTrunkSegmentBinding(model_base.BASEV2):

    """Represents binding of segments in trunk networks."""
    __tablename__ = 'cisco_n1kv_trunk_segments'

    trunk_segment_id = sa.Column(sa.String(36),
                                 sa.ForeignKey('networks.id',
                                               ondelete="CASCADE"),
                                 primary_key=True)
    segment_id = sa.Column(sa.String(36), nullable=False, primary_key=True)
    dot1qtag = sa.Column(sa.String(36), nullable=False, primary_key=True)


class N1kvMultiSegmentNetworkBinding(model_base.BASEV2):

    """Represents binding of segments in multi-segment networks."""
    __tablename__ = 'cisco_n1kv_multi_segments'

    multi_segment_id = sa.Column(sa.String(36),
                                 sa.ForeignKey('networks.id',
                                               ondelete="CASCADE"),
                                 primary_key=True)
    segment1_id = sa.Column(sa.String(36), nullable=False, primary_key=True)
    segment2_id = sa.Column(sa.String(36), nullable=False, primary_key=True)
    encap_profile_name = sa.Column(sa.String(36))



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Author: Sylvain Afchain <sylvain.afchain@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
from oslo.config import cfg

from neutron.agent.common import config
from neutron.openstack.common.notifier import test_notifier
from neutron.openstack.common import uuidutils
from neutron.services.metering.agents import metering_agent
from neutron.tests import base


_uuid = uuidutils.generate_uuid

TENANT_ID = _uuid()
LABEL_ID = _uuid()
ROUTERS = [{'status': 'ACTIVE',
            'name': 'router1',
            'gw_port_id': None,
            'admin_state_up': True,
            'tenant_id': TENANT_ID,
            '_metering_labels': [{'rules': [],
                                  'id': LABEL_ID}],
            'id': _uuid()}]


class TestMeteringOperations(base.BaseTestCase):

    def setUp(self):
        super(TestMeteringOperations, self).setUp()
        cfg.CONF.register_opts(metering_agent.MeteringAgent.Opts)
        config.register_root_helper(cfg.CONF)

        self.noop_driver = ('neutron.services.metering.drivers.noop.'
                            'noop_driver.NoopMeteringDriver')
        cfg.CONF.set_override('driver', self.noop_driver)
        cfg.CONF.set_override('measure_interval', 0)
        cfg.CONF.set_override('report_interval', 0)

        self.setup_notification_driver()

        metering_rpc = ('neutron.services.metering.agents.metering_agent.'
                        'MeteringPluginRpc._get_sync_data_metering')
        self.metering_rpc_patch = mock.patch(metering_rpc, return_value=[])
        self.metering_rpc_patch.start()

        self.driver_patch = mock.patch(self.noop_driver, autospec=True)
        self.driver_patch.start()

        loopingcall_patch = mock.patch(
            'neutron.openstack.common.loopingcall.FixedIntervalLoopingCall')
        loopingcall_patch.start()

        self.agent = metering_agent.MeteringAgent('my agent', cfg.CONF)
        self.driver = self.agent.metering_driver

    def test_add_metering_label(self):
        self.agent.add_metering_label(None, ROUTERS)
        self.assertEqual(self.driver.add_metering_label.call_count, 1)

    def test_remove_metering_label(self):
        self.agent.remove_metering_label(None, ROUTERS)
        self.assertEqual(self.driver.remove_metering_label.call_count, 1)

    def test_update_metering_label_rule(self):
        self.agent.update_metering_label_rules(None, ROUTERS)
        self.assertEqual(self.driver.update_metering_label_rules.call_count, 1)

    def test_routers_updated(self):
        self.agent.routers_updated(None, ROUTERS)
        self.assertEqual(self.driver.update_routers.call_count, 1)

    def test_get_traffic_counters(self):
        self.agent._get_traffic_counters(None, ROUTERS)
        self.assertEqual(self.driver.get_traffic_counters.call_count, 1)

    def test_notification_report(self):
        self.agent.routers_updated(None, ROUTERS)

        self.driver.get_traffic_counters.return_value = {LABEL_ID:
                                                         {'pkts': 88,
                                                          'bytes': 444}}
        self.agent._metering_loop()

        self.assertNotEqual(len(test_notifier.NOTIFICATIONS), 0)
        for n in test_notifier.NOTIFICATIONS:
            if n['event_type'] == 'l3.meter':
                break

        self.assertEqual(n['event_type'], 'l3.meter')

        payload = n['payload']
        self.assertEqual(payload['tenant_id'], TENANT_ID)
        self.assertEqual(payload['label_id'], LABEL_ID)
        self.assertEqual(payload['pkts'], 88)
        self.assertEqual(payload['bytes'], 444)

    def test_router_deleted(self):
        label_id = _uuid()
        self.driver.get_traffic_counters = mock.MagicMock()
        self.driver.get_traffic_counters.return_value = {label_id:
                                                         {'pkts': 44,
                                                          'bytes': 222}}
        self.agent._add_metering_info = mock.MagicMock()

        self.agent.routers_updated(None, ROUTERS)
        self.agent.router_deleted(None, ROUTERS[0]['id'])

        self.assertEqual(self.agent._add_metering_info.call_count, 1)
        self.assertEqual(self.driver.remove_router.call_count, 1)

        self.agent._add_metering_info.assert_called_with(label_id, 44, 222)


class TestMeteringDriver(base.BaseTestCase):
    def setUp(self):
        super(TestMeteringDriver, self).setUp()
        cfg.CONF.register_opts(metering_agent.MeteringAgent.Opts)
        config.register_root_helper(cfg.CONF)

        self.noop_driver = ('neutron.services.metering.drivers.noop.'
                            'noop_driver.NoopMeteringDriver')
        cfg.CONF.set_override('driver', self.noop_driver)

        self.agent = metering_agent.MeteringAgent('my agent', cfg.CONF)
        self.driver = mock.Mock()
        self.agent.metering_driver = self.driver

    def test_add_metering_label_with_bad_driver_impl(self):
        del self.driver.add_metering_label

        with mock.patch.object(metering_agent, 'LOG') as log:
            self.agent.add_metering_label(None, ROUTERS)
            log.exception.assert_called_with(mock.ANY,
                                             {'driver': self.noop_driver,
                                              'func': 'add_metering_label'})

    def test_add_metering_label_runtime_error(self):
        self.driver.add_metering_label.side_effect = RuntimeError

        with mock.patch.object(metering_agent, 'LOG') as log:
            self.agent.add_metering_label(None, ROUTERS)
            log.exception.assert_called_with(mock.ANY,
                                             {'driver': self.noop_driver,
                                              'func':
                                              'add_metering_label'})



# Copyright 2014 IBM Corp.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Mohammad Banikazemi, IBM Corp


import mock

from neutron.openstack.common import uuidutils
from neutron.plugins.ibm.common import constants
from neutron.plugins.ibm import sdnve_api
from neutron.tests import base

RESOURCE_PATH = {
    'network': "ln/networks/",
}
RESOURCE = 'network'
HTTP_OK = 200
TENANT_ID = uuidutils.generate_uuid()


class TestSdnveApi(base.BaseTestCase):

    def setUp(self):
        super(TestSdnveApi, self).setUp()

        class MockKeystoneClient(object):
            def __init__(self, **kwargs):
                pass

            def get_tenant_name(self, id):
                return 'test tenant name'

        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'KeystoneClient',
                        new=MockKeystoneClient):
            self.api = sdnve_api.Client()

    def mock_do_request(self, method, url, body=None, headers=None,
                        params=None, connection_type=None):
        return (HTTP_OK, url)

    def mock_do_request_tenant(self, method, url, body=None, headers=None,
                               params=None, connection_type=None):
        return (HTTP_OK, {'id': TENANT_ID,
                          'network_type': constants.TENANT_TYPE_OF})

    def mock_do_request_no_tenant(self, method, url, body=None, headers=None,
                                  params=None, connection_type=None):
        return (None, None)

    def mock_process_request(self, body):
        return body

    def test_sdnve_api_list(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request):
            result = self.api.sdnve_list(RESOURCE)
            self.assertEqual(result, (HTTP_OK, RESOURCE_PATH[RESOURCE]))

    def test_sdnve_api_show(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request):
            result = self.api.sdnve_show(RESOURCE, TENANT_ID)
            self.assertEqual(result,
                             (HTTP_OK, RESOURCE_PATH[RESOURCE] + TENANT_ID))

    def test_sdnve_api_create(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request):
            with mock.patch('neutron.plugins.ibm.sdnve_api.'
                            'Client.process_request',
                            new=self.mock_process_request):
                result = self.api.sdnve_create(RESOURCE, '')
                self.assertEqual(result, (HTTP_OK, RESOURCE_PATH[RESOURCE]))

    def test_sdnve_api_update(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request):
            with mock.patch('neutron.plugins.ibm.sdnve_api.'
                            'Client.process_request',
                            new=self.mock_process_request):
                result = self.api.sdnve_update(RESOURCE, TENANT_ID, '')
                self.assertEqual(result,
                                 (HTTP_OK,
                                  RESOURCE_PATH[RESOURCE] + TENANT_ID))

    def test_sdnve_api_delete(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request):
            result = self.api.sdnve_delete(RESOURCE, TENANT_ID)
            self.assertEqual(result,
                             (HTTP_OK, RESOURCE_PATH[RESOURCE] + TENANT_ID))

    def test_sdnve_get_tenant_by_id(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request_tenant):
            id = TENANT_ID
            result = self.api.sdnve_get_tenant_byid(id)
            self.assertEqual(result,
                             (TENANT_ID, constants.TENANT_TYPE_OF))

    def test_sdnve_check_and_create_tenant(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request_tenant):
            id = TENANT_ID
            result = self.api.sdnve_check_and_create_tenant(id)
            self.assertEqual(result, TENANT_ID)

    def test_sdnve_check_and_create_tenant_fail(self):
        with mock.patch('neutron.plugins.ibm.sdnve_api.'
                        'Client.do_request',
                        new=self.mock_do_request_no_tenant):
            id = TENANT_ID
            result = self.api.sdnve_check_and_create_tenant(
                id, constants.TENANT_TYPE_OF)
            self.assertIsNone(result)



# Copyright 2014 OneConvergence, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Test Library for OneConvergencePlugin."""

import contextlib
import uuid

import mock
from oslo.config import cfg

from neutron import context
from neutron.extensions import portbindings
from neutron.manager import NeutronManager
from neutron.plugins.oneconvergence import plugin as nvsd_plugin
from neutron.tests.unit import _test_extension_portbindings as test_bindings
from neutron.tests.unit import test_db_plugin as test_plugin
from neutron.tests.unit import test_l3_plugin

PLUGIN_NAME = 'neutron.plugins.oneconvergence.plugin.OneConvergencePluginV2'


class OneConvergencePluginV2TestCase(test_plugin.NeutronDbPluginV2TestCase):
    _plugin_name = PLUGIN_NAME

    def setUp(self):
        def mocked_oneconvergence_init(self):
            def side_effect(*args, **kwargs):
                return {'id': str(uuid.uuid4())}

            self.nvsdlib = mock.Mock()
            self.nvsdlib.create_network.side_effect = side_effect

        with mock.patch.object(nvsd_plugin.OneConvergencePluginV2,
                               'oneconvergence_init',
                               new=mocked_oneconvergence_init):
            super(OneConvergencePluginV2TestCase,
                  self).setUp(self._plugin_name)


class TestOneConvergencePluginNetworksV2(test_plugin.TestNetworksV2,
                                         OneConvergencePluginV2TestCase):
    pass


class TestOneConvergencePluginSubnetsV2(test_plugin.TestSubnetsV2,
                                        OneConvergencePluginV2TestCase):
    def test_update_subnet_inconsistent_ipv6_gatewayv4(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_create_subnet_with_v6_allocation_pool(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_inconsistent_ipv6_hostroute_dst_v4(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_inconsistent_ipv6_hostroute_np_v4(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_create_subnet_ipv6_attributes(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_create_subnet_ipv6_single_attribute_set(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_ipv6_attributes(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_ipv6_inconsistent_enable_dhcp(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_ipv6_inconsistent_ra_attribute(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_update_subnet_ipv6_inconsistent_address_attribute(self):
        self.skipTest("NVSD Plugin does not support IPV6.")


class TestOneConvergencePluginPortsV2(test_plugin.TestPortsV2,
                                      test_bindings.PortBindingsTestCase,
                                      OneConvergencePluginV2TestCase):
    VIF_TYPE = portbindings.VIF_TYPE_OVS

    def test_requested_subnet_id_v4_and_v6(self):
        self.skipTest("NVSD Plugin does not support IPV6.")

    def test_port_vif_details(self):
        plugin = NeutronManager.get_plugin()
        with self.port(name='name') as port1:
            ctx = context.get_admin_context()
            port = plugin.get_port(ctx, port1['port']['id'])
            self.assertEqual(port['binding:vif_type'],
                             portbindings.VIF_TYPE_OVS)

    def test_ports_vif_details(self):
        cfg.CONF.set_default('allow_overlapping_ips', True)
        plugin = NeutronManager.get_plugin()
        with contextlib.nested(self.port(), self.port()) as (port1, port2):
            ctx = context.get_admin_context()
            ports = plugin.get_ports(ctx)
            self.assertEqual(len(ports), 2)
            for port in ports:
                self.assertEqual(port['binding:vif_type'],
                                 portbindings.VIF_TYPE_OVS)


class TestOneConvergenceBasicGet(test_plugin.TestBasicGet,
                                 OneConvergencePluginV2TestCase):
    pass


class TestOneConvergenceV2HTTPResponse(test_plugin.TestV2HTTPResponse,
                                       OneConvergencePluginV2TestCase):
    pass


class TestOneConvergenceL3NatTestCase(test_l3_plugin.L3NatDBIntTestCase):
    _plugin_name = PLUGIN_NAME

    def setUp(self):
        def mocked_oneconvergence_init(self):
            def side_effect(*args, **kwargs):
                return {'id': str(uuid.uuid4())}

            self.nvsdlib = mock.Mock()
            self.nvsdlib.create_network.side_effect = side_effect

        ext_mgr = test_l3_plugin.L3TestExtensionManager()

        with mock.patch.object(nvsd_plugin.OneConvergencePluginV2,
                               'oneconvergence_init',
                               new=mocked_oneconvergence_init):
            super(TestOneConvergenceL3NatTestCase,
                  self).setUp(plugin=self._plugin_name, ext_mgr=ext_mgr)

    def test_floatingip_with_invalid_create_port(self):
        self._test_floatingip_with_invalid_create_port(self._plugin_name)



# Copyright 2014 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from neutron.common import constants
from neutron.common import exceptions
from neutron.openstack.common import excutils
from neutron.openstack.common import log
from neutron.plugins.vmware.common import utils
from neutron.plugins.vmware.nsxlib import _build_uri_path
from neutron.plugins.vmware.nsxlib import do_request
from neutron.plugins.vmware.nsxlib import format_exception
from neutron.plugins.vmware.nsxlib import get_all_query_pages

HTTP_GET = "GET"
HTTP_POST = "POST"
HTTP_DELETE = "DELETE"
HTTP_PUT = "PUT"

SECPROF_RESOURCE = "security-profile"

LOG = log.getLogger(__name__)


def mk_body(**kwargs):
    """Convenience function creates and dumps dictionary to string.

    :param kwargs: the key/value pirs to be dumped into a json string.
    :returns: a json string.
    """
    return json.dumps(kwargs, ensure_ascii=False)


def query_security_profiles(cluster, fields=None, filters=None):
    return get_all_query_pages(
        _build_uri_path(SECPROF_RESOURCE,
                        fields=fields,
                        filters=filters),
        cluster)


def create_security_profile(cluster, tenant_id, neutron_id, security_profile):
    """Create a security profile on the NSX backend.

    :param cluster: a NSX cluster object reference
    :param tenant_id: identifier of the Neutron tenant
    :param neutron_id: neutron security group identifier
    :param security_profile: dictionary with data for
    configuring the NSX security profile.
    """
    path = "/ws.v1/security-profile"
    # Allow all dhcp responses and all ingress traffic
    hidden_rules = {'logical_port_egress_rules':
                    [{'ethertype': 'IPv4',
                      'protocol': constants.PROTO_NUM_UDP,
                      'port_range_min': constants.DHCP_RESPONSE_PORT,
                      'port_range_max': constants.DHCP_RESPONSE_PORT,
                      'ip_prefix': '0.0.0.0/0'}],
                    'logical_port_ingress_rules':
                    [{'ethertype': 'IPv4'},
                     {'ethertype': 'IPv6'}]}
    display_name = utils.check_and_truncate(security_profile.get('name'))
    # NOTE(salv-orlando): neutron-id tags are prepended with 'q' for
    # historical reasons
    body = mk_body(
        tags=utils.get_tags(os_tid=tenant_id, q_sec_group_id=neutron_id),
        display_name=display_name,
        logical_port_ingress_rules=(
            hidden_rules['logical_port_ingress_rules']),
        logical_port_egress_rules=hidden_rules['logical_port_egress_rules']
    )
    rsp = do_request(HTTP_POST, path, body, cluster=cluster)
    if security_profile.get('name') == 'default':
        # If security group is default allow ip traffic between
        # members of the same security profile is allowed and ingress traffic
        # from the switch
        rules = {'logical_port_egress_rules': [{'ethertype': 'IPv4',
                                                'profile_uuid': rsp['uuid']},
                                               {'ethertype': 'IPv6',
                                                'profile_uuid': rsp['uuid']}],
                 'logical_port_ingress_rules': [{'ethertype': 'IPv4'},
                                                {'ethertype': 'IPv6'}]}

        update_security_group_rules(cluster, rsp['uuid'], rules)
    LOG.debug(_("Created Security Profile: %s"), rsp)
    return rsp


def update_security_group_rules(cluster, spid, rules):
    path = "/ws.v1/security-profile/%s" % spid

    # Allow all dhcp responses in
    rules['logical_port_egress_rules'].append(
        {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP,
         'port_range_min': constants.DHCP_RESPONSE_PORT,
         'port_range_max': constants.DHCP_RESPONSE_PORT,
         'ip_prefix': '0.0.0.0/0'})
    # If there are no ingress rules add bunk rule to drop all ingress traffic
    if not rules['logical_port_ingress_rules']:
        rules['logical_port_ingress_rules'].append(
            {'ethertype': 'IPv4', 'ip_prefix': '127.0.0.1/32'})
    try:
        body = mk_body(
            logical_port_ingress_rules=rules['logical_port_ingress_rules'],
            logical_port_egress_rules=rules['logical_port_egress_rules'])
        rsp = do_request(HTTP_PUT, path, body, cluster=cluster)
    except exceptions.NotFound as e:
        LOG.error(format_exception("Unknown", e, locals()))
        #FIXME(salvatore-orlando): This should not raise NeutronException
        raise exceptions.NeutronException()
    LOG.debug(_("Updated Security Profile: %s"), rsp)
    return rsp


def update_security_profile(cluster, spid, name):
    return do_request(HTTP_PUT,
                      _build_uri_path(SECPROF_RESOURCE, resource_id=spid),
                      json.dumps({
                          "display_name": utils.check_and_truncate(name)
                      }),
                      cluster=cluster)


def delete_security_profile(cluster, spid):
    path = "/ws.v1/security-profile/%s" % spid

    try:
        do_request(HTTP_DELETE, path, cluster=cluster)
    except exceptions.NotFound:
        with excutils.save_and_reraise_exception():
            # This is not necessarily an error condition
            LOG.warn(_("Unable to find security profile %s on NSX backend"),
                     spid)



# Copyright (c) 2013 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Sylvain Afchain, eNovance SAS
# @author: Francois Eleouet, Orange
# @author: Mathieu Rohon, Orange

from oslo.config import cfg

from neutron.common import constants as const
from neutron import context as n_context
from neutron.db import api as db_api
from neutron.openstack.common import log as logging
from neutron.plugins.ml2 import driver_api as api
from neutron.plugins.ml2.drivers.l2pop import config  # noqa
from neutron.plugins.ml2.drivers.l2pop import db as l2pop_db
from neutron.plugins.ml2.drivers.l2pop import rpc as l2pop_rpc

LOG = logging.getLogger(__name__)


class L2populationMechanismDriver(api.MechanismDriver,
                                  l2pop_db.L2populationDbMixin):

    def initialize(self):
        LOG.debug(_("Experimental L2 population driver"))
        self.rpc_ctx = n_context.get_admin_context_without_session()
        self.migrated_ports = {}
        self.deleted_ports = {}

    def _get_port_fdb_entries(self, port):
        return [[port['mac_address'],
                 ip['ip_address']] for ip in port['fixed_ips']]

    def delete_port_precommit(self, context):
        # TODO(matrohon): revisit once the original bound segment will be
        # available in delete_port_postcommit. in delete_port_postcommit
        # agent_active_ports will be equal to 0, and the _update_port_down
        # won't need agent_active_ports_count_for_flooding anymore
        port_context = context.current
        fdb_entries = self._update_port_down(context, port_context, 1)
        self.deleted_ports[context.current['id']] = fdb_entries

    def delete_port_postcommit(self, context):
        fanout_msg = self.deleted_ports.pop(context.current['id'], None)
        if fanout_msg:
            l2pop_rpc.L2populationAgentNotify.remove_fdb_entries(
                self.rpc_ctx, fanout_msg)

    def _get_diff_ips(self, orig, port):
        orig_ips = set([ip['ip_address'] for ip in orig['fixed_ips']])
        port_ips = set([ip['ip_address'] for ip in port['fixed_ips']])

        # check if an ip has been added or removed
        orig_chg_ips = orig_ips.difference(port_ips)
        port_chg_ips = port_ips.difference(orig_ips)

        if orig_chg_ips or port_chg_ips:
            return orig_chg_ips, port_chg_ips

    def _fixed_ips_changed(self, context, orig, port, diff_ips):
        orig_ips, port_ips = diff_ips

        port_infos = self._get_port_infos(context, orig)
        if not port_infos:
            return
        agent, agent_ip, segment, port_fdb_entries = port_infos

        orig_mac_ip = [[port['mac_address'], ip] for ip in orig_ips]
        port_mac_ip = [[port['mac_address'], ip] for ip in port_ips]

        upd_fdb_entries = {port['network_id']: {agent_ip: {}}}

        ports = upd_fdb_entries[port['network_id']][agent_ip]
        if orig_mac_ip:
            ports['before'] = orig_mac_ip

        if port_mac_ip:
            ports['after'] = port_mac_ip

        l2pop_rpc.L2populationAgentNotify.update_fdb_entries(
            self.rpc_ctx, {'chg_ip': upd_fdb_entries})

        return True

    def update_port_postcommit(self, context):
        port = context.current
        orig = context.original

        diff_ips = self._get_diff_ips(orig, port)
        if diff_ips:
            self._fixed_ips_changed(context, orig, port, diff_ips)
        if (port['binding:host_id'] != orig['binding:host_id']
            and port['status'] == const.PORT_STATUS_ACTIVE
            and not self.migrated_ports.get(orig['id'])):
            # The port has been migrated. We have to store the original
            # binding to send appropriate fdb once the port will be set
            # on the destination host
            self.migrated_ports[orig['id']] = orig
        elif port['status'] != orig['status']:
            if port['status'] == const.PORT_STATUS_ACTIVE:
                self._update_port_up(context)
            elif port['status'] == const.PORT_STATUS_DOWN:
                fdb_entries = self._update_port_down(context, port)
                l2pop_rpc.L2populationAgentNotify.remove_fdb_entries(
                    self.rpc_ctx, fdb_entries)
            elif port['status'] == const.PORT_STATUS_BUILD:
                orig = self.migrated_ports.pop(port['id'], None)
                if orig:
                    # this port has been migrated : remove its entries from fdb
                    fdb_entries = self._update_port_down(context, orig)
                    l2pop_rpc.L2populationAgentNotify.remove_fdb_entries(
                        self.rpc_ctx, fdb_entries)

    def _get_port_infos(self, context, port):
        agent_host = port['binding:host_id']
        if not agent_host:
            return

        session = db_api.get_session()
        agent = self.get_agent_by_host(session, agent_host)
        if not agent:
            return

        agent_ip = self.get_agent_ip(agent)
        if not agent_ip:
            LOG.warning(_("Unable to retrieve the agent ip, check the agent "
                          "configuration."))
            return

        segment = context.bound_segment
        if not segment:
            LOG.warning(_("Port %(port)s updated by agent %(agent)s "
                          "isn't bound to any segment"),
                        {'port': port['id'], 'agent': agent})
            return

        tunnel_types = self.get_agent_tunnel_types(agent)
        if segment['network_type'] not in tunnel_types:
            return

        fdb_entries = self._get_port_fdb_entries(port)

        return agent, agent_ip, segment, fdb_entries

    def _update_port_up(self, context):
        port_context = context.current
        port_infos = self._get_port_infos(context, port_context)
        if not port_infos:
            return
        agent, agent_ip, segment, port_fdb_entries = port_infos

        agent_host = port_context['binding:host_id']
        network_id = port_context['network_id']

        session = db_api.get_session()
        agent_active_ports = self.get_agent_network_active_port_count(
            session, agent_host, network_id)

        other_fdb_entries = {network_id:
                             {'segment_id': segment['segmentation_id'],
                              'network_type': segment['network_type'],
                              'ports': {agent_ip: []}}}

        if agent_active_ports == 1 or (
                self.get_agent_uptime(agent) < cfg.CONF.l2pop.agent_boot_time):
            # First port activated on current agent in this network,
            # we have to provide it with the whole list of fdb entries
            agent_fdb_entries = {network_id:
                                 {'segment_id': segment['segmentation_id'],
                                  'network_type': segment['network_type'],
                                  'ports': {}}}
            ports = agent_fdb_entries[network_id]['ports']

            network_ports = self.get_network_ports(session, network_id)
            for network_port in network_ports:
                binding, agent = network_port
                if agent.host == agent_host:
                    continue

                ip = self.get_agent_ip(agent)
                if not ip:
                    LOG.debug(_("Unable to retrieve the agent ip, check "
                                "the agent %(agent_host)s configuration."),
                              {'agent_host': agent.host})
                    continue

                agent_ports = ports.get(ip, [const.FLOODING_ENTRY])
                agent_ports += self._get_port_fdb_entries(binding.port)
                ports[ip] = agent_ports

            # And notify other agents to add flooding entry
            other_fdb_entries[network_id]['ports'][agent_ip].append(
                const.FLOODING_ENTRY)

            if ports.keys():
                l2pop_rpc.L2populationAgentNotify.add_fdb_entries(
                    self.rpc_ctx, agent_fdb_entries, agent_host)

        # Notify other agents to add fdb rule for current port
        other_fdb_entries[network_id]['ports'][agent_ip] += port_fdb_entries

        l2pop_rpc.L2populationAgentNotify.add_fdb_entries(self.rpc_ctx,
                                                          other_fdb_entries)

    def _update_port_down(self, context, port_context,
                          agent_active_ports_count_for_flooding=0):
        port_infos = self._get_port_infos(context, port_context)
        if not port_infos:
            return
        agent, agent_ip, segment, port_fdb_entries = port_infos

        agent_host = port_context['binding:host_id']
        network_id = port_context['network_id']

        session = db_api.get_session()
        agent_active_ports = self.get_agent_network_active_port_count(
            session, agent_host, network_id)

        other_fdb_entries = {network_id:
                             {'segment_id': segment['segmentation_id'],
                              'network_type': segment['network_type'],
                              'ports': {agent_ip: []}}}
        if agent_active_ports == agent_active_ports_count_for_flooding:
            # Agent is removing its last activated port in this network,
            # other agents needs to be notified to delete their flooding entry.
            other_fdb_entries[network_id]['ports'][agent_ip].append(
                const.FLOODING_ENTRY)
        # Notify other agents to remove fdb rules for current port
        other_fdb_entries[network_id]['ports'][agent_ip] += port_fdb_entries

        return other_fdb_entries



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit Tests for linuxbridge rpc
"""

import fixtures
from oslo.config import cfg

from neutron.agent import rpc as agent_rpc
from neutron.common import topics
from neutron.openstack.common import context
from neutron.plugins.linuxbridge import lb_neutron_plugin as plb
from neutron.tests import base


class rpcApiTestCase(base.BaseTestCase):
    def _test_lb_api(self, rpcapi, topic, method, rpc_method,
                     expected_msg=None, **kwargs):
        ctxt = context.RequestContext('fake_user', 'fake_project')
        expected_retval = 'foo' if method == 'call' else None
        if not expected_msg:
            expected_msg = rpcapi.make_msg(method, **kwargs)
        expected_msg['version'] = rpcapi.BASE_RPC_API_VERSION
        if rpc_method == 'cast' and method == 'run_instance':
            kwargs['call'] = False

        self.fake_args = None
        self.fake_kwargs = None

        def _fake_rpc_method(*args, **kwargs):
            self.fake_args = args
            self.fake_kwargs = kwargs
            if expected_retval:
                return expected_retval

        self.useFixture(fixtures.MonkeyPatch(
            'neutron.openstack.common.rpc.' + rpc_method, _fake_rpc_method))

        retval = getattr(rpcapi, method)(ctxt, **kwargs)

        self.assertEqual(expected_retval, retval)
        expected_args = [ctxt, topic, expected_msg]

        for arg, expected_arg in zip(self.fake_args, expected_args):
            self.assertEqual(expected_arg, arg)

    def test_delete_network(self):
        rpcapi = plb.AgentNotifierApi(topics.AGENT)
        self._test_lb_api(rpcapi,
                          topics.get_topic_name(topics.AGENT,
                                                topics.NETWORK,
                                                topics.DELETE),
                          'network_delete', rpc_method='fanout_cast',
                          network_id='fake_request_spec')

    def test_port_update(self):
        cfg.CONF.set_override('rpc_support_old_agents', False, 'AGENT')
        rpcapi = plb.AgentNotifierApi(topics.AGENT)
        expected_msg = rpcapi.make_msg('port_update',
                                       port='fake_port',
                                       network_type='vlan',
                                       physical_network='fake_net',
                                       segmentation_id='fake_vlan_id')
        self._test_lb_api(rpcapi,
                          topics.get_topic_name(topics.AGENT,
                                                topics.PORT,
                                                topics.UPDATE),
                          'port_update', rpc_method='fanout_cast',
                          expected_msg=expected_msg,
                          port='fake_port',
                          physical_network='fake_net',
                          vlan_id='fake_vlan_id')

    def test_port_update_old_agent(self):
        cfg.CONF.set_override('rpc_support_old_agents', True, 'AGENT')
        rpcapi = plb.AgentNotifierApi(topics.AGENT)
        expected_msg = rpcapi.make_msg('port_update',
                                       port='fake_port',
                                       network_type='vlan',
                                       physical_network='fake_net',
                                       segmentation_id='fake_vlan_id',
                                       vlan_id='fake_vlan_id')
        self._test_lb_api(rpcapi,
                          topics.get_topic_name(topics.AGENT,
                                                topics.PORT,
                                                topics.UPDATE),
                          'port_update', rpc_method='fanout_cast',
                          expected_msg=expected_msg,
                          port='fake_port',
                          physical_network='fake_net',
                          vlan_id='fake_vlan_id')

    def test_device_details(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_lb_api(rpcapi, topics.PLUGIN,
                          'get_device_details', rpc_method='call',
                          device='fake_device',
                          agent_id='fake_agent_id')

    def test_update_device_down(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_lb_api(rpcapi, topics.PLUGIN,
                          'update_device_down', rpc_method='call',
                          device='fake_device',
                          agent_id='fake_agent_id',
                          host='fake_host')

    def test_update_device_up(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_lb_api(rpcapi, topics.PLUGIN,
                          'update_device_up', rpc_method='call',
                          device='fake_device',
                          agent_id='fake_agent_id',
                          host='fake_host')



# Copyright (c) 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo.config import cfg


ml2_opts = [
    cfg.ListOpt('type_drivers',
                default=['local', 'flat', 'vlan', 'gre', 'vxlan'],
                help=_("List of network type driver entrypoints to be loaded "
                       "from the neutron.ml2.type_drivers namespace.")),
    cfg.ListOpt('tenant_network_types',
                default=['local'],
                help=_("Ordered list of network_types to allocate as tenant "
                       "networks.")),
    cfg.ListOpt('mechanism_drivers',
                default=[],
                help=_("An ordered list of networking mechanism driver "
                       "entrypoints to be loaded from the "
                       "neutron.ml2.mechanism_drivers namespace.")),
]


cfg.CONF.register_opts(ml2_opts, "ml2")
